/***
*
*  Getsys.prg
*
*  Standard Clipper 5.2 GET/READ Subsystem
*
*  Copyright (c) 1991-1993, Computer Associates International, Inc.
*  All rights reserved.
*
*
*	Modified by:	John M. Skelton, Dec-92.
*			Added support for ReadModeless().
*
*	Modified by:	John M. Skelton, May-93.
*			Added support for GETs in dialogs.
*
*	Copyright (C) 1993 Skelton Software, Kendal Cottage, Hillam, Leeds, UK.
*	All Rights Reserved.
*
*  This version adds the following public functions:
*
*     ReadKill( [<lKill>] )       --> lKill
*     ReadUpdated( [<lUpdated>] ) --> lUpdated
*     ReadFormat( [<bFormat>] )   --> bFormat | NIL
*
*  NOTE: compile with /m /n /w
*
*/

#define	WIN_WANT_ALL
#include "windows.ch"
#include "accel.ch"
#include "Set.ch"
#include "Inkey.ch"
#include "Getexit.ch"

#define	MAKELPARAM(nLow, nHigh)		(nLow + (nHigh * 65536))

        // test if <nValue> has any of the bits in <nMask> set
#xtranslate Test( <nValue>, <nMask> )                                  ;
        => (C4W_And( <nValue>, <nMask> ) != 0)

#define K_UNDO          K_CTRL_U


//
// State variables for active READ
//
STATIC sbFormat
STATIC slUpdated := .F.
STATIC slKillRead
STATIC slBumpTop
STATIC slBumpBot
STATIC snLastExitState
STATIC snLastPos
STATIC soActiveGet
STATIC scReadProcName
STATIC snReadProcLine

//
// Clip-4-Win state variables
//
STATIC	aGetLists := {}, aStates := {}
STATIC	nEvent
STATIC	aGetList
STATIC	nGetPos
STATIC	nPosClick
STATIC	shDlg
STATIC	shWnd
STATIC	saInfo
STATIC	snId


//
// Format of array used to preserve state variables
//
#define GSV_KILLREAD       1
#define GSV_BUMPTOP        2
#define GSV_BUMPBOT        3
#define GSV_LASTEXIT       4
#define GSV_LASTPOS        5
#define GSV_ACTIVEGET      6
#define GSV_READVAR        7
#define GSV_READPROCNAME   8
#define GSV_READPROCLINE   9

#define GSV_COUNT          9


//
// Format of array used to preserve state variables for ReadModeless()
// and DReadModeless()
//
#define ST_KILLREAD       1
#define ST_BUMPTOP        2
#define ST_BUMPBOT        3
#define ST_LASTEXIT       4
#define ST_LASTPOS        5
#define ST_ACTIVEGET      6
#define ST_READVAR        7
#define ST_READPROCNAME   8
#define ST_READPROCLINE   9
#define ST_UPDATED        10
#define ST_POS            11
#define	ST_HDLG           12
#define	ST_HWND           13
#define	ST_INFO           14
#define	ST_ID             15

#define ST_COUNT          15


#ifdef	WANT_BEEPS
#define	MayBeep()	MessageBeep()
#else
#define	MayBeep()
#endif	// WANT_BEEPS



FUNCTION IsDReadOK(hDlg, nId)
LOCAL i, aInfo
IF (i := ascan(aStates, {|a| a[ST_HDLG] == hDlg})) == 0
   RETURN .t.
ENDIF
aInfo := aStates[i][ST_INFO]
DReadModeless(0, 0, WM_CHAR, K_PGDN, 0, aGetLists[i], 0)
IF aGetList == nil
//   aeval(aInfo, {|nId| UnSubClsWindow(GetDlgItem(hDlg, nId))})
ENDIF
RETURN (aGetList == nil)


PROCEDURE DReadCancel(hDlg)
LOCAL i, aInfo
IF (i := ascan(aStates, {|a| a[ST_HDLG] == hDlg})) == 0
   RETURN
ENDIF
aInfo := aStates[i][ST_INFO]
DReadModeless(0, 0, WM_CHAR, K_ESC, 0, aGetLists[i], 0)
//aeval(aInfo, {|nId| UnSubClsWindow(GetDlgItem(hDlg, nId))})
RETURN


PROCEDURE DReadSetInfo(hDlg, nId, GetList, nGPos)
LOCAL nPos, nElem, oGet := GetList[nGPos], cBuf

IF (nElem := ascan(aGetLists, {|a| a == GetList})) == 0
   // a new GetList
   DReadModeless(0, GetDlgItem(hDlg, nId), WM_NULL, 0, 0, GetList, nGPos)
   nElem := ascan(aGetLists, {|a| a == GetList})
ENDIF
RestoreState(nElem, @nPos)

IF nGPos > len(saInfo)
   asize(saInfo, nGPos)
ENDIF

saInfo[nGPos] := snId := nId

// fetch initial value, as chars
cBuf := transform(oGet:varGet(),				;
                  iif(oGet:picture == nil, "", oGet:picture))

// you can remove the next statement if you don't want to limit the
// length of the user's input
SendDlgItemMessage(hDlg, nId, EM_LIMITTEXT, len(cBuf), 0)

// you can remove the next statement if you don't like to see the initial
// values of all the GETs (you'll still see the current GET and any that
// have been completed)
SendDlgItemMessage(hDlg, nId, WM_SETTEXT, 0, cBuf)

SaveState(nElem, nPos)

RETURN


PROCEDURE DReadSetFont(hFont)
SendDlgItemMessage(shDlg, snId, WM_SETFONT, hFont, 1)
RETURN


/***
*   DRangeCheck()
*
*   NOTE: unused second param for 5.00 compatibility.
*/

FUNCTION DRangeCheck(oGet, junk, lo, hi)
LOCAL cMsg, xValue, hWnd

   IF ( !oGet:changed )             // This may not be a good idea...
      RETURN (.T.)
   ENDIF

   xValue := oGet:VarGet()

   IF ( xValue >= lo .and. xValue <= hi )
      RETURN (.T.)
   ENDIF

   cMsg := "Range: " + Ltrim(Transform(lo, "")) + ;
           " - " + Ltrim(Transform(hi, ""))

   hWnd := GetDlgItem(shDlg, snId)
   MessageBox(hWnd, cMsg, "Error", MB_ICONSTOP)
   IF GetFocus() != hWnd
      SetFocus(hWnd)
   ENDIF

RETURN (.F.)



/***
*   DReadModeless()
*   Standard modeless READ on an array of GETs in a dialog.
*
*   Called by the dialog when necessary.
*
*   Returns: 1 if a message was processed, 0 otherwise.
*
*   You are expected to use this with code like this:
*
*      FUNCTION MyDialog(hWndDlg, nMsg, nwParam, nlParam)
*      STATIC GetList
*      STATIC cVar
*
*      DO CASE
*      CASE nMsg == WM_INITDIALOG
*
*         GetList := {}
*         cVar := space(10)
*         @ DIALOG hWndDlg ID ID_MYEDIT   ;
*           GET cVar PICTURE "@!"
*         
*/

FUNCTION DReadModeless(nProc, hWnd, nMsg, nwParam, nlParam, GetList, nGPos)
   LOCAL oGet
   LOCAL nPos
   LOCAL j, nElem, nRet

   // set these for use in SET KEYs
   scReadProcName := ProcName(1)
   snReadProcLine := ProcLine(1)

   // find GetList in aGetLists, or add it if missing,
   // and set context (state) variables
   IF (nElem := ascan(aGetLists, {|a| a == GetList})) == 0

      IF nMsg != WM_NULL
         RETURN 0
      ENDIF

      // a new GetList

      IF ( ValType(sbFormat) == "B" )
         Eval(sbFormat)
      ENDIF

      IF ( Empty(GetList) )
         RETURN 0
      ENDIF

      // save GetList and set state
      IF (nElem := ascan(aGetLists, nil)) == 0
         aadd(aGetLists, GetList)
         nElem := len(aGetLists)
         aadd(aStates, array(ST_COUNT))
      ELSE
         aGetLists[nElem] := GetList
      ENDIF
      slKillRead := .f.
      slBumpTop := .f.
      slBumpBot := .f.
      snLastExitState := GE_DOWN    // was 0
      snLastPos := 0
      GetActive( NIL )
      ReadVar( "" )
      scReadProcName := ""
      snReadProcLine := 0
      slUpdated := .f.
      shDlg := GetParent(hWnd)
      shWnd := hWnd
      saInfo := {}

      // set initial GET to be read
      nEvent := EVENT_NONE      // don't really care
      nPos := Settle( GetList, 0 )
      SaveState(nElem, nPos)
      IF nMsg == WM_NULL
         RETURN 0
      ENDIF
   ELSE
      // restore state variables
      RestoreState(nElem, @nPos)
   ENDIF

   // deal with any events that can be done without the get reader
   IF nPos == 0
      nRet := 0
   ELSEIF valtype(GetList[nPos]:reader) != "B"
      // using standard reader, so can do some events now
      DO CASE
      CASE nMsg == WM_SETFOCUS
         nPosClick := nGetPos := nGPos
         aGetList := GetList
         oGet := GetList[nPos]
         IF oGet:exitState != GE_NOEXIT
            nPos := nGPos
         ENDIF
      CASE nMsg == WM_KILLFOCUS
         shWnd := hWnd
         nRet := 0
      CASE nMsg == WM_LBUTTONDOWN
         // check the mouse position
         IF (nPosClick := nGPos) == nil
            // not a click in any of the GetList
            RETURN 0   // wait for the next event
         ENDIF

         // getting here means it's a click on a GET
         // (the current one if nGPos == nPos)
         nGetPos := nGPos
         aGetList := GetList
      ENDCASE
   ENDIF

   WHILE ( nPos <> 0 )

      // get next GET from list and post it as the active GET
      oGet := GetList[nPos]
      PostActiveGet( oGet )
      snId := saInfo[nPos]


      // read the GET
      IF ( ValType( oGet:reader ) == "B" )
         IF nMsg == WM_KEYDOWN .and. nwParam < 48
            // various special keys (VK_* in accel.ch)
            // you might like to convert some of them to
            // the K_* values used by Clipper
         ENDIF
         // passing the GetList may encourage bad coding,
         // but may also be extremely useful
         nRet := Eval( oGet:reader, oGet, GetList, hWnd, nMsg, nwParam, nlParam )    // use custom reader block
      ELSE
         // be careful not to WM_SETFOCUS unless the GET
         // can cope with it
         IF oGet:exitState == GE_NOEXIT         ;
            .or. nMsg != WM_SETFOCUS            ;
            .or. (nMsg == WM_SETFOCUS .and. nPos == nGPos)
            nRet := DGetModeless( oGet, hWnd, nMsg, nwParam, nlParam )      // use standard reader
         ELSE
            // fallthrough
         ENDIF
      ENDIF

      IF oGet:exitState == GE_NOEXIT ;
         .or. (oGet:exitState == GE_WHEN ;
               .and. (nMsg == WM_SETFOCUS .or. nMsg == WM_KILLFOCUS))
         // save state ready for the next event
         SaveState(nElem, nPos)

         RETURN nRet      // wait for the next event
      ENDIF

      // move to next GET based on exit condition
      nPos := Settle( GetList, nPos )
      nEvent := EVENT_NONE      // cancel any mouse click

      IF nPos <> 0
         // arrange to do no more than select the GET

         // get next GET from list and post it as the active GET
         oGet := GetList[nPos]
         PostActiveGet( oGet )
         snId := saInfo[nPos]

         IF nMsg == WM_SETFOCUS
            // read the GET
            IF ( ValType( oGet:reader ) == "B" )
               // passing the GetList may encourage bad coding,
               // but may also be extremely useful
               nRet := Eval( oGet:reader, oGet, GetList, hWnd, nMsg, nwParam, nlParam )    // use custom reader block
            ELSE
               nRet := DGetModeless( oGet, hWnd, nMsg, nwParam, nlParam )      // use standard reader
            ENDIF
         ELSEIF nMsg != WM_KILLFOCUS
            SetFocus(GetDlgItem(shDlg, saInfo[nPos]))
         ENDIF

         // save state ready for the next event
         SaveState(nElem, nPos)
         RETURN nRet      // wait for the next event
      ENDIF
   END

   // end of this GetList

   //////////
   //
   // Because we've finished with the GetList, but the dialog may
   // still be on the screen, we want to stop the user mistakenly
   // trying to alter the values.  There's a choice of ways below.
   //
   //////////

//#define DISABLE_WHEN_DONE	// You can choose this, if you like
#ifdef DISABLE_WHEN_DONE
   // disable all the GETs
   aeval(saInfo, {|nId| EnableWindow(GetDlgItem(shDlg, nId), .f.)})
#else
   // set all the GETs read only
   aeval(saInfo, {|nId| SendDlgItemMessage(shDlg, nId, EM_SETREADONLY, 1, 0)})
   // make sure the caret is hidden
   // aeval(saInfo, {|nId| SendDlgItemMessage(shDlg, nId, WM_KILLFOCUS, 0, 0)})
   aeval(saInfo, {|nId| PostMessage(GetDlgItem(shDlg, nId), WM_KILLFOCUS, 0, 0)})
#endif // DISABLE_WHEN_DONE

   // to help the user, try to set the Windows focus to an OK button
   // (note that IDOK is defined in windows.ch)
   IF (j := GetDlgItem(shDlg, IDOK)) != 0
      SetFocus(j)
   ENDIF

   // we're done with this GetList, so remove it and its state
   aGetList := nil
   adel(aGetLists, nElem)
   asize(aGetLists, len(aGetLists) - 1)
   adel(aStates, nElem)
   asize(aStates, len(aStates) - 1)

RETURN nRet


STATIC PROCEDURE DGetApplyKey( hWnd, oGet, nKey )
GetApplyKey( oGet, nKey )
RETURN


STATIC PROCEDURE DUpdateEdit( hWnd, oGet )
LOCAL j
IF hWnd == 0
   RETURN
ENDIF
SendMessage(hWnd, WM_SETTEXT, 0, oGet:buffer)
j := oGet:pos - oGet:col
SendMessage(hWnd, EM_SETSEL, 1, MAKELPARAM(j, j))
SendMessage(hWnd, EM_SETMODIFY, 0, 0)
RETURN


STATIC PROCEDURE DKeyDown( oGet, hWnd, nMsg, nwParam, nlParam )
LOCAL j, nKey
LOCAL lCtrl := Test(GetKeyState(VK_CONTROL), 32768)  // .t. if pressed

DO CASE
CASE nwParam == VK_SHIFT
   nKey := 0   // ignore
CASE nwParam == VK_CONTROL
   nKey := 0   // ignore
CASE nwParam == VK_CLEAR
   // maybe something ?
CASE nwParam == VK_PRIOR
   nKey := K_PGUP
CASE nwParam == VK_NEXT
//   nKey := K_PGDN              // can do this if you wish
   nKey := 0                     // ignore - best to use an OK button
CASE nwParam == VK_END
//#define CTRL_END_SPECIAL
#ifdef CTRL_END_SPECIAL
   nKey := iif(lCtrl, K_CTRL_END, K_END)
#else
   nKey := iif(lCtrl, 0, K_END)  // ignore Ctrl-End: best to use an OK button
#endif
CASE nwParam == VK_HOME
   nKey := iif(lCtrl, K_CTRL_HOME, K_HOME)
CASE nwParam == VK_LEFT
   nKey := iif(lCtrl, K_CTRL_LEFT, K_LEFT)
CASE nwParam == VK_UP
   nKey := K_UP
CASE nwParam == VK_RIGHT
   nKey := iif(lCtrl, K_CTRL_RIGHT, K_RIGHT)
CASE nwParam == VK_DOWN
   nKey := K_DOWN
CASE nwParam == VK_INSERT
   nKey := K_INS
CASE nwParam == VK_DELETE
   nKey := K_DEL
ENDCASE

IF nKey != nil
   IF nKey != 0
      DGetApplyKey(hWnd, oGet, nKey)
      DUpdateEdit(hWnd, oGet)
   ENDIF
ENDIF

RETURN



/***
*   DGetModeless()
*   Standard modeless read of a single GET in a dialog.
*/
FUNCTION DGetModeless( oGet, hWnd, nMsg, nwParam, nlParam )
LOCAL nRet := 1

   // read the GET if the WHEN condition is satisfied
   IF oGet:exitState == GE_NOEXIT .or. ( GetPreValidate(oGet) )

      // activate the GET for reading
      IF !oGet:hasFocus
         oGet:SetFocus()
      ENDIF

      WHILE ( oGet:exitState == GE_NOEXIT )

         // check for initial typeout (no editable positions)
         IF !oGet:badDate() .and. ( oGet:typeOut ) .and. nMsg == WM_CHAR .and. nwParam != K_BS
            IF ! SET( _SET_CONFIRM )
               oGet:exitState := GE_ENTER
               DUpdateEdit( hWnd, oGet )
            ENDIF
         ENDIF

         // apply keystrokes until exit
         WHILE ( oGet:exitState == GE_NOEXIT )
            DO CASE
            CASE nMsg == WM_SETFOCUS
               IF oGet == aGetList[nGetPos]
                  // focus to current GET
                  DUpdateEdit( hWnd, oGet )
               ELSE
                  IF nGetPos < ascan(aGetList, {|oG| oG == oGet})
                     oGet:exitState := GE_UP
                  ELSE
                     oGet:exitState := GE_DOWN
                  ENDIF
   
                  // disallow exit if the VALID condition is not satisfied
                  IF ( !GetPostValidate(oGet) )
                     oGet:exitState := GE_NOEXIT
                     MayBeep()
                     SetFocus(GetDlgItem(shDlg, snId))   // grab focus back
                     RETURN 1          // wait for another event
                  ENDIF

                  nEvent := EVENT_LCLICK
               ENDIF
               nRet := 0
            CASE nMsg == WM_KILLFOCUS
               shWnd := hWnd
               nRet := 0
            CASE nMsg == WM_KEYDOWN
               // only handle the special keys  (see accel.ch)
               IF nwParam < 48 .and. nwParam != VK_BACK
                  DKeyDown( oGet, hWnd, nMsg, nwParam, nlParam )
               ENDIF
            CASE nMsg == WM_CHAR
               IF hWnd == 0
                  hWnd := GetDlgItem(shDlg, snId)
               ENDIF
               IF nwParam == K_ESC
                  __KillRead()
               ENDIF
               DGetApplyKey( hWnd, oGet, nwParam )
               DUpdateEdit( hWnd, oGet )
            CASE nMsg == WM_DESTROY
               IF oGet:hasFocus
                  DGetApplyKey( hWnd, oGet, K_ESC )
               ENDIF
               nRet := 0
            CASE nMsg == WM_SYSCOMMAND
               IF nwParam == SC_CLOSE
                  IF oGet:hasFocus
                     DGetApplyKey( hWnd, oGet, K_ESC )
                  ENDIF
                  nRet := 0
               ENDIF
            CASE nMsg == WM_LBUTTONDOWN
               IF oGet == aGetList[nGetPos]
                  // current GET, track the position
                  oGet:pos = oGet:col + C4W_LoWord(SendMessage(hWnd, EM_GETSEL, 0, 0))
                  DUpdateEdit( hWnd, oGet )
               ELSE
                  // let SetFocus sort it out
               ENDIF
               nRet := 0
            ENDCASE

            IF oGet:exitState == GE_NOEXIT
               RETURN nRet         // wait for another event
            ENDIF
         END

         // disallow exit if the VALID condition is not satisfied
         IF !oGet:hasFocus
            oGet:SetFocus()
         ENDIF
         IF ( !GetPostValidate(oGet) )
            oGet:exitState := GE_NOEXIT
            MayBeep()
            RETURN nRet       // wait for another event
         ENDIF

      END

      // de-activate the GET
      oGet:KillFocus()
      // update the display - need oGet:buffer, so use SetFocus/KillFocus
      oGet:SetFocus()
      IF nMsg == WM_SETFOCUS
         DUpdateEdit( shWnd, oGet )
      ELSE
         DUpdateEdit( hWnd, oGet )
      ENDIF
      oGet:KillFocus()

   ELSEIF oGet:exitState == GE_WHEN .and. nMsg == WM_SETFOCUS
      MayBeep()
      IF oGet == aGetList[nGetPos]
         // focus to current GET
         // attempt to prevent focus problems
//         EnableWindow(hWnd, .f.)
      ENDIF

      // failed to select the GET
      // to help the user, try to set the Windows focus to an OK or Cancel button
      // (note that IDOK and IDCANCEL are defined in windows.ch)
      IF GetDlgItem(shDlg, IDOK) != 0
         SetFocus(GetDlgItem(shDlg, IDOK))
      ELSEIF GetDlgItem(shDlg, IDCANCEL) != 0
         SetFocus(GetDlgItem(shDlg, IDCANCEL))
      // ELSE we're stuck with the focus
      ENDIF
      IF IsWindowEnabled(shWnd)
//         SetFocus(shWnd)      // set focus back
      ENDIF
   ENDIF

RETURN nRet



/***
*  ReadModeless()
*  Standard modeless READ on an array of GETs.
*
*  Returns: NIL until read is done, then the value of Updated.
*
*
*  You are expected to use this with code like this:
*
*     nHand := AddHandler(hWnd, {|nEvent| MyReadModeless(GetList, nEvent, nHand)})
*
*     . . .
*
*     PROCEDURE MyReadModeless(GetList, nEvent, nHand)
*     LOCAL lUpdated
*
*     IF (lUpdated := ReadModeless(GetList, nEvent)) == NIL
*        // not yet done
*        RETURN            // back to event loop
*     ENDIF
*
*     // the read has completed (or been abandoned)
*     DelHandler(nHand)    // if you've finished
*
*     . . .
*
*     RETURN               // back to event loop
*/
FUNCTION ReadModeless( GetList, nEv )

   LOCAL oGet
   LOCAL nPos
   LOCAL i, hWnd, hOldWnd, nRow, nCol

   nEvent := nEv

   // set these for use in SET KEYs
   scReadProcName := ProcName(1)
   snReadProcLine := ProcLine(1)

   IF nEvent != EVENT_DESTROY
      // make sure the right window is selected
      hOldWnd := SelectWindow(hWnd := _LasthWnd())
   ENDIF

   // find GetList in aGetLists, or add it if missing,
   // and set context (state) variables
   IF (i := ascan(aGetLists, {|a| a == GetList})) == 0

      // a new GetList

      IF ( ValType(sbFormat) == "B" )
         Eval(sbFormat)
      ENDIF

      IF ( Empty(GetList) )
         // S87 compat.
         SetPos( MaxRow()-1, 0 )
         SelectWindow(hOldWnd)
         RETURN (.f.)         // NOTE
      ENDIF

      // save GetList and set state
      aadd(aGetLists, GetList)
      i := len(aGetLists)
      aadd(aStates, array(ST_COUNT))
      // add an event handler, so we can clean up
      AddHandler(hWnd,                                    ;
                 {|nEvent| IIF(nEvent == EVENT_DESTROY,   ;
                               Cleanup(GetList), ) } )
      slKillRead := .f.
      slBumpTop := .f.
      slBumpBot := .f.
      snLastExitState := 0
      snLastPos := 0
      GetActive( NIL )
      ReadVar( "" )
      scReadProcName := ""
      snReadProcLine := 0
      slUpdated := .f.

      // set initial GET to be read
      nPos := Settle( GetList, 0 )
      SaveState(i, nPos)
   ELSE
      // restore state variables
      RestoreState(i, @nPos)
   ENDIF

   // deal with any events that can be done without the get reader
   IF nPos != 0 .and. valtype(GetList[nPos]:reader) != "B"
      // using standard reader, so can do some events now
      DO CASE
      CASE nEvent == EVENT_REDRAW
         // re-draw the GetList
         aeval(GetList, {|oGet| oGet:Display()})
         IF nPos != 0
            // display the current GET last, so the caret
            // is in the right place
            GetList[nPos]:Display()
         ENDIF
         SelectWindow(hOldWnd)
         RETURN nil      // wait for the next event
      CASE nEvent == EVENT_DESTROY
         nPos := 0      // simply want to exit ReadModeless()
      CASE nEvent == EVENT_LCLICK
         // check the mouse position
         IF (nPosClick := PosInGetList(GetList, MouseRow(), MouseCol())) == nil
            // not a click in any of the GetList
            SelectWindow(hOldWnd)
            RETURN nil   // wait for the next event
         ENDIF

         // getting here means it's a click on a GET
         // (the current one if MouseRow() == oGet:row)
      ENDCASE
   ENDIF

   WHILE ( nPos != 0 )

      // get next GET from list and post it as the active GET
      oGet := GetList[nPos]
      PostActiveGet( oGet )


      // read the GET
      IF ( ValType( oGet:reader ) == "B" )
         // passing the GetList may encourage bad coding,
         // but may also be extremely useful
         Eval( oGet:reader, oGet, nEvent, GetList )    // use custom reader block
         IF nEvent == EVENT_LCLICK
            // oGet:exitState determines if this is used
            nPosClick := PosInGetList(GetList, MouseRow(), MouseCol())
         ENDIF
      ELSE
         GetModeless( oGet, nEvent )      // use standard reader
      ENDIF

      IF oGet:exitState == GE_NOEXIT
         // save state ready for the next event
         SaveState(i, nPos)
         SelectWindow(hOldWnd)
         RETURN nil      // wait for the next event
      ENDIF

      // move to next GET based on exit condition
      nPos := Settle( GetList, nPos )

      IF nPos != 0
         // arrange to do no more than display the GET
         nEvent := EVENT_NONE
      ENDIF
   END

   // we're done with this GetList, so remove ie it and its state
   adel(aGetLists, i)
   asize(aGetLists, len(aGetLists) - 1)
   adel(aStates, i)
   asize(aStates, len(aStates) - 1)

   IF nEvent != EVENT_DESTROY
      // S87 compat.
      SetPos( MaxRow()-1, 0 )

      // restore the current window
      SelectWindow(hOldWnd)
   ENDIF

RETURN (slUpdated)



/***
*   PosInGetList()
*   Find which GET in a GetList includes position nRow,nCol.
*
*   May be called by custom GET readers.
*/
FUNCTION PosInGetList(GetList, nRow, nCol)
LOCAL i, n := len(GetList)
LOCAL oGet, nWidth

FOR i := 1 TO n
   oGet := GetList[i]
   IF nRow != oGet:row .or. nCol < oGet:col
      LOOP         // not this GET
   ENDIF
   nWidth := len(transform(oGet:varGet(),      ;
                           IIF(oGet:picture == nil, "", oGet:picture)))
   IF nCol >= oGet:col .and. nCol < oGet:col + nWidth
      // move to this GET
      RETURN i
   ENDIF
NEXT i

RETURN nil



/***
*   Cleanup()
*   Take a GET list off the active list.
*
*/
STATIC PROCEDURE Cleanup(GetList)
LOCAL i

IF (i := ascan(aGetLists, {|a| a == GetList})) != 0

   // we're done with this GetList, so remove it and its state
   adel(aGetLists, i)
   asize(aGetLists, len(aGetLists) - 1)
   adel(aStates, i)
   asize(aStates, len(aStates) - 1)

ENDIF

RETURN



/***
*   SaveState()
*   Save the state of the current GET list.
*
*/
STATIC PROCEDURE SaveState(i, nPos)

LOCAL saved := aStates[i]

   saved[ ST_KILLREAD ] := slKillRead

   saved[ ST_BUMPTOP ] := slBumpTop

   saved[ ST_BUMPBOT ] := slBumpBot

   saved[ ST_LASTEXIT ] := snLastExitState

   saved[ ST_LASTPOS ] := snLastPos

   saved[ ST_ACTIVEGET ] := GetActive( NIL )

   saved[ ST_READVAR ] := ReadVar( "" )

   saved[ ST_READPROCNAME ] := scReadProcName

   saved[ ST_READPROCLINE ] := snReadProcLine

   saved[ ST_UPDATED ] := slUpdated

   saved[ ST_POS ] := nPos

   saved[ ST_HDLG ] := shDlg

   saved[ ST_HWND ] := shWnd

   saved[ ST_INFO ] := saInfo

   saved[ ST_ID ] := snId
RETURN



/***
*   RestoreState()
*   Restore the state of a GET list.
*
*/
STATIC PROCEDURE RestoreState(i, nPos)
LOCAL saved := aStates[i]

   slKillRead := saved[ ST_KILLREAD ]

   slBumpTop := saved[ ST_BUMPTOP ]

   slBumpBot := saved[ ST_BUMPBOT ]

   snLastExitState := saved[ ST_LASTEXIT ]

   snLastPos := saved[ ST_LASTPOS ]

   GetActive( saved[ ST_ACTIVEGET ] )

   ReadVar( saved[ ST_READVAR ] )

   scReadProcName := saved[ ST_READPROCNAME ]

   snReadProcLine := saved[ ST_READPROCLINE ]

   slUpdated := saved[ ST_UPDATED ]

   nPos := saved[ ST_POS ]

   shDlg := saved[ ST_HDLG ]

   shWnd := saved[ ST_HWND ]

   saInfo := saved[ ST_INFO ]

   snId := saved[ ST_ID ]
RETURN



/***
*   GetModeless()
*   Standard modeless read of a single GET.
*/
PROCEDURE GetModeless( oGet, nEvent )

   // read the GET if the WHEN condition is satisfied
   IF oGet:exitState == GE_NOEXIT .or. ( GetPreValidate(oGet) )

      // activate the GET for reading
      IF !oGet:hasFocus
         oGet:SetFocus()
      ENDIF

      WHILE ( oGet:exitState == GE_NOEXIT )

         // check for initial typeout (no editable positions)
         IF ( oGet:typeOut )
            oGet:exitState := GE_ENTER
         ENDIF

         // apply keystrokes until exit
         WHILE ( oGet:exitState == GE_NOEXIT )
            DO CASE
            CASE nEvent == EVENT_KEY
               GetApplyKey( oGet, Inkey(0) )
            CASE nEvent == EVENT_LCLICK
               IF MouseRow() == oGet:row
                  // current GET, so change the position
                  oGet:pos -= col() - MouseCol()
                  oGet:Display()
               ELSEIF MouseRow() < oGet:row
                  oGet:exitState := GE_UP
               ELSE
                  oGet:exitState := GE_DOWN
               ENDIF
               
            ENDCASE

            IF oGet:exitState == GE_NOEXIT
               RETURN   // wait for another event
            ENDIF
         END

         // disallow exit if the VALID condition is not satisfied
         IF ( !GetPostValidate(oGet) )
            oGet:exitState := GE_NOEXIT
            RETURN   // wait for another event
         ENDIF

      END

      // de-activate the GET
      oGet:KillFocus()

   ENDIF

RETURN



/***
*
*  ReadModal()
*
*  Standard modal READ on an array of GETs
*
*/
FUNCTION ReadModal( GetList, nPos )

   LOCAL oGet
   LOCAL aSavGetSysVars

   IF ( VALTYPE( sbFormat ) == "B" )
      EVAL( sbFormat )
   ENDIF

   IF ( EMPTY( GetList ) )
      
      // S'87 compatibility
      SETPOS( MAXROW() - 1, 0 )
      RETURN (.F.)                  // NOTE

   ENDIF

   // Preserve state variables
   aSavGetSysVars := ClearGetSysVars()

   // Set these for use in SET KEYs
   scReadProcName := PROCNAME( 1 )
   snReadProcLine := PROCLINE( 1 )

   // Set initial GET to be read
   IF !( VALTYPE( nPos ) == "N" .AND. nPos > 0 )
      nPos := Settle( Getlist, 0 )
   ENDIF

   WHILE !( nPos == 0 )

      // Get next GET from list and post it as the active GET
      PostActiveGet( oGet := GetList[ nPos ] )

      // Read the GET
      IF ( VALTYPE( oGet:reader ) == "B" )
         EVAL( oGet:reader, oGet )    // Use custom reader block
      ELSE
         GetReader( oGet )            // Use standard reader
      ENDIF

      // Move to next GET based on exit condition
      nPos := Settle( GetList, nPos )

   ENDDO


   // Restore state variables
   RestoreGetSysVars( aSavGetSysVars )

   // S'87 compatibility
   SETPOS( MAXROW() - 1, 0 )

   RETURN ( slUpdated )



/***
*
*  GetReader()
*
*  Standard modal read of a single GET
*
*/
PROCEDURE GetReader( oGet )

   // Read the GET if the WHEN condition is satisfied
   IF ( GetPreValidate( oGet ) )

      // Activate the GET for reading
      oGet:setFocus()

      WHILE ( oGet:exitState == GE_NOEXIT )

         // Check for initial typeout (no editable positions)
         IF ( oGet:typeOut )
            oGet:exitState := GE_ENTER
         ENDIF

         // Apply keystrokes until exit
         WHILE ( oGet:exitState == GE_NOEXIT )
            GetApplyKey( oGet, inkey( 0 ) )
         ENDDO

         // Disallow exit if the VALID condition is not satisfied
         IF ( !GetPostValidate( oGet ) )
            oGet:exitState := GE_NOEXIT
         ENDIF
      ENDDO

      // De-activate the GET
      oGet:killFocus()

   ENDIF

   RETURN



/***
*
*  GetApplyKey()
*
*  Apply a single INKEY() keystroke to a GET
*
*  NOTE: GET must have focus.
*
*/
PROCEDURE GetApplyKey( oGet, nKey )

   LOCAL cKey
   LOCAL bKeyBlock

   // Check for SET KEY first
   IF !( ( bKeyBlock := setkey( nKey ) ) == NIL )
      GetDoSetKey( bKeyBlock, oGet )
      RETURN                           // NOTE
   ENDIF

   DO CASE
   CASE ( nKey == K_UP )
      oGet:exitState := GE_UP

   CASE ( nKey == K_SH_TAB )
      oGet:exitState := GE_UP

   CASE ( nKey == K_DOWN )
      oGet:exitState := GE_DOWN

   CASE ( nKey == K_TAB )
      oGet:exitState := GE_DOWN

   CASE ( nKey == K_ENTER )
      oGet:exitState := GE_ENTER

   CASE ( nKey == K_ESC )
      IF ( SET( _SET_ESCAPE ) )
         
         oGet:undo()
         oGet:exitState := GE_ESCAPE

      ENDIF

   CASE ( nKey == K_PGUP )
      oGet:exitState := GE_WRITE

   CASE ( nKey == K_PGDN )
      oGet:exitState := GE_WRITE

   CASE ( nKey == K_CTRL_HOME )
      oGet:exitState := GE_TOP


#ifdef CTRL_END_SPECIAL

   // Both ^W and ^End go to the last GET
   CASE ( nKey == K_CTRL_END )
      oGet:exitState := GE_BOTTOM

#else

   // Both ^W and ^End terminate the READ (the default)
   CASE ( nKey == K_CTRL_W )
      oGet:exitState := GE_WRITE

#endif


   CASE ( nKey == K_INS )
      SET( _SET_INSERT, !SET( _SET_INSERT ) )
      ShowScoreboard()

   CASE ( nKey == K_UNDO )
      oGet:undo()

   CASE ( nKey == K_HOME )
      oGet:home()

   CASE ( nKey == K_END )
      oGet:end()

   CASE ( nKey == K_RIGHT )
      oGet:right()

   CASE ( nKey == K_LEFT )
      oGet:left()

   CASE ( nKey == K_CTRL_RIGHT )
      oGet:wordRight()

   CASE ( nKey == K_CTRL_LEFT )
      oGet:wordLeft()

   CASE ( nKey == K_BS )
      oGet:backSpace()

   CASE ( nKey == K_DEL )
      oGet:delete()

   CASE ( nKey == K_CTRL_T )
      oGet:delWordRight()

   CASE ( nKey == K_CTRL_Y )
      oGet:delEnd()

   CASE ( nKey == K_CTRL_BS )
      oGet:delWordLeft()

   OTHERWISE

      IF ( nKey >= 32 .AND. nKey <= 255 )

         cKey := CHR( nKey )

         IF ( oGet:type == "N" .AND. ( cKey == "." .OR. cKey == "," ) )
            oGet:toDecPos()
         ELSE
            
            IF ( SET( _SET_INSERT ) )
               oGet:insert( cKey )
            ELSE
               oGet:overstrike( cKey )
            ENDIF

            IF ( oGet:typeOut )
               IF ( SET( _SET_BELL ) )
                  ?? CHR(7)
               ENDIF

               IF ( !SET( _SET_CONFIRM ) )
                  oGet:exitState := GE_ENTER
               ENDIF
            ENDIF

         ENDIF

      ENDIF

   ENDCASE

   RETURN



/***
*
*  GetPreValidate()
*
*  Test entry condition (WHEN clause) for a GET
*
*/
FUNCTION GetPreValidate( oGet )

   LOCAL lSavUpdated
   LOCAL lWhen := .T.

   IF !( oGet:preBlock == NIL )

      lSavUpdated := slUpdated

      lWhen := EVAL( oGet:preBlock, oGet )

      oGet:display()

      ShowScoreBoard()
      slUpdated := lSavUpdated

   ENDIF

   IF ( slKillRead )
      
      lWhen := .F.
      oGet:exitState := GE_ESCAPE       // Provokes ReadModal() exit

   ELSEIF ( !lWhen )
      
      oGet:exitState := GE_WHEN         // Indicates failure

   ELSE
      
      oGet:exitState := GE_NOEXIT       // Prepares for editing

   END

   RETURN ( lWhen )



/***
*
*  GetPostValidate()
*
*  Test exit condition (VALID clause) for a GET
*
*  NOTE: Bad dates are rejected in such a way as to preserve edit buffer
*
*/
FUNCTION GetPostValidate( oGet )

   LOCAL lSavUpdated
   LOCAL lValid := .T.


   IF ( oGet:exitState == GE_ESCAPE )
      RETURN ( .T. )                   // NOTE
   ENDIF

   IF ( oGet:badDate() )
      oGet:home()
      DateMsg()
      ShowScoreboard()
      RETURN ( .F. )                   // NOTE
   ENDIF

   // If editing occurred, assign the new value to the variable
   IF ( oGet:changed )
      oGet:assign()
      slUpdated := .T.
   ENDIF

   // Reform edit buffer, set cursor to home position, redisplay
   oGet:reset()

   // Check VALID condition if specified
   IF !( oGet:postBlock == NIL )

      lSavUpdated := slUpdated

      // S'87 compatibility
      SETPOS( oGet:row, oGet:col + LEN( oGet:buffer ) )

      lValid := EVAL( oGet:postBlock, oGet )

      // Reset S'87 compatibility cursor position
      SETPOS( oGet:row, oGet:col )

      ShowScoreBoard()
      oGet:updateBuffer()

      slUpdated := lSavUpdated

      IF ( slKillRead )
         oGet:exitState := GE_ESCAPE      // Provokes ReadModal() exit
         lValid := .T.

      ENDIF
   ENDIF

   RETURN ( lValid )



/***
*
*  GetDoSetKey()
*
*  Process SET KEY during editing
*
*/
PROCEDURE GetDoSetKey( keyBlock, oGet )

   LOCAL lSavUpdated

   // If editing has occurred, assign variable
   IF ( oGet:changed )
      oGet:assign()
      slUpdated := .T.
   ENDIF

   lSavUpdated := slUpdated

   EVAL( keyBlock, scReadProcName, snReadProcLine, ReadVar() )

   ShowScoreboard()
   oGet:updateBuffer()

   slUpdated := lSavUpdated

   IF ( slKillRead )
      oGet:exitState := GE_ESCAPE      // provokes ReadModal() exit
   ENDIF

   RETURN





/***
*              READ services
*/



/***
*
*  Settle()
*
*  Returns new position in array of Get objects, based on:
*     - current position
*     - exitState of Get object at current position
*
*  NOTES: return value of 0 indicates termination of READ
*         exitState of old Get is transferred to new Get
*
*/
STATIC FUNCTION Settle( GetList, nPos )

   LOCAL nExitState

   IF ( nPos == 0 )
      nExitState := GE_DOWN
   ELSE
      nExitState := GetList[ nPos ]:exitState
   ENDIF

   IF ( nExitState == GE_ESCAPE .or. nExitState == GE_WRITE )
      RETURN ( 0 )               // NOTE
   ENDIF

   IF !( nExitState == GE_WHEN )
      // Reset state info
      snLastPos := nPos
      slBumpTop := .F.
      slBumpBot := .F.
   ELSE
      // Re-use last exitState, do not disturb state info
      nExitState := snLastExitState
   ENDIF

   //
   // Move
   //
   DO CASE
   CASE nEvent == EVENT_LCLICK
      // position according to the mouse
      nPos := nPosClick

   CASE ( nExitState == GE_UP )
      nPos--

   CASE ( nExitState == GE_DOWN )
      nPos++

   CASE ( nExitState == GE_TOP )
      nPos       := 1
      slBumpTop  := .T.
      nExitState := GE_DOWN

   CASE ( nExitState == GE_BOTTOM )
      nPos       := LEN( GetList )
      slBumpBot  := .T.
      nExitState := GE_UP

   CASE ( nExitState == GE_ENTER )
      nPos++

   ENDCASE

   //
   // Bounce
   //
   IF ( nPos == 0 )                       // Bumped top
      IF ( !ReadExit() .and. !slBumpBot )
         slBumpTop  := .T.
         nPos       := snLastPos
         nExitState := GE_DOWN
      ENDIF

   ELSEIF ( nPos == len( GetList ) + 1 )  // Bumped bottom
      IF ( !ReadExit() .and. !( nExitState == GE_ENTER ) .and. !slBumpTop )
         slBumpBot  := .T.
         nPos       := snLastPos
         nExitState := GE_UP
      ELSE
         nPos := 0
      ENDIF
   ENDIF

   // Record exit state
   snLastExitState := nExitState

   IF !( nPos == 0 )
      GetList[ nPos ]:exitState := nExitState
   ENDIF
   
   RETURN ( nPos )



/***
*
*  PostActiveGet()
*
*  Post active GET for ReadVar(), GetActive()
*
*/
STATIC PROCEDURE PostActiveGet( oGet )

   GetActive( oGet )
   ReadVar( GetReadVar( oGet ) )

   ShowScoreBoard()

   RETURN



/***
*
*  ClearGetSysVars()
*
*  Save and clear READ state variables. Return array of saved values
*
*  NOTE: 'Updated' status is cleared but not saved (S'87 compatibility)
*/
STATIC FUNCTION ClearGetSysVars()

   LOCAL aSavSysVars[ GSV_COUNT ]

   // Save current sys vars
   aSavSysVars[ GSV_KILLREAD ]     := slKillRead
   aSavSysVars[ GSV_BUMPTOP ]      := slBumpTop
   aSavSysVars[ GSV_BUMPBOT ]      := slBumpBot
   aSavSysVars[ GSV_LASTEXIT ]     := snLastExitState
   aSavSysVars[ GSV_LASTPOS ]      := snLastPos
   aSavSysVars[ GSV_ACTIVEGET ]    := GetActive( NIL )
   aSavSysVars[ GSV_READVAR ]      := ReadVar( "" )
   aSavSysVars[ GSV_READPROCNAME ] := scReadProcName
   aSavSysVars[ GSV_READPROCLINE ] := snReadProcLine

   // Re-init old ones
   slKillRead      := .F.
   slBumpTop       := .F.
   slBumpBot       := .F.
   snLastExitState := 0
   snLastPos       := 0
   scReadProcName  := ""
   snReadProcLine  := 0
   slUpdated       := .F.

   RETURN ( aSavSysVars )



/***
*
*  RestoreGetSysVars()
*
*  Restore READ state variables from array of saved values
*
*  NOTE: 'Updated' status is not restored (S'87 compatibility)
*
*/
STATIC PROCEDURE RestoreGetSysVars( aSavSysVars )

   slKillRead      := aSavSysVars[ GSV_KILLREAD ]
   slBumpTop       := aSavSysVars[ GSV_BUMPTOP ]
   slBumpBot       := aSavSysVars[ GSV_BUMPBOT ]
   snLastExitState := aSavSysVars[ GSV_LASTEXIT ]
   snLastPos       := aSavSysVars[ GSV_LASTPOS ]

   GetActive( aSavSysVars[ GSV_ACTIVEGET ] )

   ReadVar( aSavSysVars[ GSV_READVAR ] )

   scReadProcName  := aSavSysVars[ GSV_READPROCNAME ]
   snReadProcLine  := aSavSysVars[ GSV_READPROCLINE ]

   RETURN



/***
*
*  GetReadVar()
*
*  Set READVAR() value from a GET
*
*/
STATIC FUNCTION GetReadVar( oGet )

   LOCAL cName := UPPER( oGet:name )
   LOCAL i

   // The following code includes subscripts in the name returned by
   // this FUNCTIONtion, if the get variable is an array element
   //
   // Subscripts are retrieved from the oGet:subscript instance variable
   //
   // NOTE: Incompatible with Summer 87
   //
   IF !( oGet:subscript == NIL )
      FOR i := 1 TO LEN( oGet:subscript )
         cName += "[" + LTRIM( STR( oGet:subscript[i] ) ) + "]"
      NEXT
   END

   RETURN ( cName )





/***
*              System Services
*/



/***
*
*  __SetFormat()
*  
*  SET FORMAT service
*
*/
PROCEDURE __SetFormat( b )
   sbFormat := IF( VALTYPE( b ) == "B", b, NIL )
   RETURN



/***
*
*  __KillRead()
*
*  CLEAR GETS service
*
*/
PROCEDURE __KillRead()
   slKillRead := .T.
   RETURN



/***
*
*  GetActive()
*
*  Retrieves currently active GET object
*/
FUNCTION GetActive( g )

   LOCAL oldActive := soActiveGet

   IF ( PCOUNT() > 0 )
      soActiveGet := g
   ENDIF

   RETURN ( oldActive )



/***
*
*  Updated()
*
*/
FUNCTION Updated()
   RETURN slUpdated



/***
*
*  ReadExit()
*
*/
FUNCTION ReadExit( lNew )
   RETURN ( SET( _SET_EXIT, lNew ) )



/***
*
*  ReadInsert()
*
*/
FUNCTION ReadInsert( lNew )
   RETURN ( SET( _SET_INSERT, lNew ) )



/***
*              Wacky Compatibility Services
*/


// Display coordinates for SCOREBOARD
#define SCORE_ROW      0
#define SCORE_COL      60


/***
*
*  ShowScoreboard()
*
*/
STATIC PROCEDURE ShowScoreboard()

   LOCAL nRow
   LOCAL nCol

   IF ( SET( _SET_SCOREBOARD ) )
      nRow := ROW()
      nCol := COL()

      SETPOS( SCORE_ROW, SCORE_COL )
      DISPOUT( IF( SET( _SET_INSERT ), "Ins", "   " ) )
      SETPOS( nRow, nCol )
   ENDIF

   RETURN



/***
*
*  DateMsg()
*
*/
STATIC PROCEDURE DateMsg()

   LOCAL nRow
   LOCAL nCol

   IF ( SET( _SET_SCOREBOARD ) )
      
      nRow := ROW()
      nCol := COL()

      SETPOS( SCORE_ROW, SCORE_COL )
      DISPOUT( "Invalid Date" )
      SETPOS( nRow, nCol )

      WHILE ( NEXTKEY() == 0 )
      END

      SETPOS( SCORE_ROW, SCORE_COL )
      DISPOUT( SPACE( 12 ) )
      SETPOS( nRow, nCol )

   ELSE

      alert("Invalid Date")

   ENDIF

   RETURN



/***
*
*  RangeCheck()
*
*  NOTE: Unused second param for 5.00 compatibility.
*
*/
FUNCTION RangeCheck( oGet, junk, lo, hi )

   LOCAL cMsg, nRow, nCol
   LOCAL xValue

   IF ( !oGet:changed )
      RETURN ( .T. )          // NOTE
   ENDIF

   xValue := oGet:varGet()

   IF ( xValue >= lo .and. xValue <= hi )
      RETURN ( .T. )          // NOTE
   ENDIF

   IF ( SET(_SET_SCOREBOARD) )
      
      cMsg := "Range: " + LTRIM( TRANSFORM( lo, "" ) ) + ;
            " - " + LTRIM( TRANSFORM( hi, "" ) )

      IF ( LEN( cMsg ) > MAXCOL() )
         cMsg := SUBSTR( cMsg, 1, MAXCOL() )
      ENDIF

      nRow := ROW()
      nCol := COL()

      SETPOS( SCORE_ROW, MIN( 60, MAXCOL() - LEN( cMsg ) ) )
      DISPOUT( cMsg )
      SETPOS( nRow, nCol )

      WHILE ( NEXTKEY() == 0 )
      END

      SETPOS( SCORE_ROW, MIN( 60, MAXCOL() - LEN( cMsg ) ) )
      DISPOUT( SPACE( LEN( cMsg ) ) )
      SETPOS( nRow, nCol )

   ELSE

      alert("Range: " + Ltrim(Transform(lo, "")) + ;
            " - " + Ltrim(Transform(hi, "")) )

   ENDIF

   RETURN ( .F. )



/***
*
*  ReadKill()
*
*/
FUNCTION ReadKill( lKill )

   LOCAL lSavKill := slKillRead

   IF ( PCOUNT() > 0 )
      slKillRead := lKill
   ENDIF

   RETURN ( lSavKill )



/***
*
*  ReadUpdated()
*
*/
FUNCTION ReadUpdated( lUpdated )
   
   LOCAL lSavUpdated := slUpdated
   
   IF ( PCOUNT() > 0 )
      slUpdated := lUpdated
   ENDIF

   RETURN ( lSavUpdated )
      


/***
*
*  ReadFormat()
*
*/
FUNCTION ReadFormat( b )
   
   LOCAL bSavFormat := sbFormat

   IF ( PCOUNT() > 0 )
      sbFormat := b
   ENDIF

   RETURN ( bSavFormat )
      
