////////////////////////////
//
//	Clip-4-Win accelerator definitions.
//
//	Copyright (C) 1993 Skelton Software, Kendal Cottage, Hillam, Leeds, UK.
//	All Rights Reserved.
//
//	Example:
//
//	#include "accel.ch"
//
//	// NOTE: You *must* use id's greater than about 500.
//	#define	ID_SPECIAL	1000
//	#define	ID_ANOK		1001
//	#define	ID_HELP		1002
//	#define	ID_SECRET	1003
//
//	ACCELERATORS
//
//	AKEY	"b",	ID_SPECIAL, ALT			// Alt-b
//	AKEY	"B",	ID_SPECIAL, ALT			// Alt-Shift-b (Alt-B)
//	AKEY	"^B",	ID_ANOK				// Control-B
//	AKEY	VK_F1,	ID_HELP,    VIRTKEY		// F1
//	AKEY	VK_F6,	ID_SECRET,  VIRTKEY, CONTROL, ALT  // Alt-Ctrl-F6
//
//	USE ACCELERATORS
//
//	END ACCELERATORS
//
////////////////////////////
//
//	Note: AccelKeyList is used as a scratch variable below.
//            may wish to declare it as LOCAL in the function where
//            you are setting up your accelerators to avoid compiler
//            warnings.  (However, such warnings are harmless.)
//
////////////////////////////

#ifndef	C4W_ACCEL_CH
#define	C4W_ACCEL_CH


#xcommand	ACCELERATORS [ <var> ]	=>	if .t.	;
						; AccelKeyList := {}

		// you can have several AKEY commands:
#xcommand	AKEY <key> , <id>		;
			[ , <v: VIRTKEY> ]	;
			[ , <c: CONTROL> ]	;
			[ , <s: SHIFT>	 ]	;
			[ , <a: ALT>	 ]	;
		=>				;
		aadd( AccelKeyList, { <key>, <id>, <.v.>, <.c.>, <.s.>, <.a.> } )

#xcommand	USE ACCELERATORS	=>	SetAccelerator(AccelKeyList)	;
						; AccelKeyList := {}

#xcommand	END ACCELERATORS	=>	endif


// Virtual key codes:	(not all exist on all computers)
#define VK_LBUTTON	1
#define VK_RBUTTON	2
#define VK_CANCEL	3		// control-break (?)
#define VK_MBUTTON      4
#define VK_BACK 	8		// backspace
#define VK_TAB		9
#define VK_CLEAR	12
#define VK_RETURN	13
#define VK_SHIFT	16
#define VK_CONTROL	17
#define VK_MENU 	18		// alt
#define VK_PAUSE	19
#define VK_CAPITAL	20		// caps lock
#define VK_ESCAPE	27
#define VK_SPACE	32
#define VK_PRIOR	33		// page up
#define VK_NEXT 	34		// page down
#define VK_END		35
#define VK_HOME 	36
#define VK_LEFT 	37
#define VK_UP		38
#define VK_RIGHT	39
#define VK_DOWN 	40
#define VK_SELECT	41
#define VK_PRINT	42
#define VK_EXECUTE	43
#define VK_SNAPSHOT	44		// print screen
#define VK_INSERT	45
#define VK_DELETE	46
#define VK_HELP 	47
#define VK_NUMPAD0	96
#define VK_NUMPAD1	97
#define VK_NUMPAD2	98
#define VK_NUMPAD3	99
#define VK_NUMPAD4	100
#define VK_NUMPAD5	101
#define VK_NUMPAD6	102
#define VK_NUMPAD7	103
#define VK_NUMPAD8	104
#define VK_NUMPAD9	105
#define VK_MULTIPLY	106
#define VK_ADD		107
#define VK_SEPARATOR	108
#define VK_SUBTRACT	109
#define VK_DECIMAL	110
#define VK_DIVIDE	111
#define VK_F1		112
#define VK_F2		113
#define VK_F3		114
#define VK_F4		115
#define VK_F5		116
#define VK_F6		117
#define VK_F7		118
#define VK_F8		119
#define VK_F9		120
#define VK_F10		121
#define VK_F11		122
#define VK_F12		123
#define VK_F13		124
#define VK_F14		125
#define VK_F15		126
#define VK_F16		127
#define VK_F17		128
#define VK_F18		129
#define VK_F19		130
#define VK_F20		131
#define VK_F21		132
#define VK_F22		133
#define VK_F23		134
#define VK_F24		135
#define VK_NUMLOCK	144
#define VK_SCROLL       145

// VK_A to VK_Z are the same as asc('A') to asc('Z')
// VK_0 to VK_9 are the same as asc('0') to asc('9')

#endif	// C4W_ACCEL_CH

