////////////////
//
//	topclass - class definitions
//
// Copyright (C) 1994 Skelton Software, Kendal Cottage, Hillam, Leeds LS25 5HP, UK.
// All Rights Reserved.
//
////////////////


#ifndef	TOPCLASS_CH
#define	TOPCLASS_CH

// Definitions for external use:  e.g. for TC_IVarType() / TC_MethodType()

#define	TC_NFALSE	0	// numeric false: <= TC_NFALSE means false

#define TC_EXPORTED	1
#define	TC_PROTECTED	2
#define	TC_READONLY	3
#define	TC_HIDDEN	4
#define	TC_ACCESS_ASSIGN 5
#define	TC_ACCESS	6
#define	TC_ASSIGN	7

#define	TC_METHOD	1	// from TC_MethodType()


// Internals:

#xtranslate _IF_(.T., [<t>], [<f>])	=> <t>
#xtranslate _IF_(.F., [<t>], [<f>])	=> <f>

#define	_TC_PUBLIC	0
//#define _TC_EXPORTED	0
#define	_TC_PROTECTED	1
#define	_TC_READONLY	2
#define	_TC_IV_HIDDEN	3

//#define _TC_PUBLIC	0	// *must* be same value as for ivar
//#define _TC_PROTECTED	1	// *must* be same value as for ivar
#define	_TC_DEFERRED	2
#define	_TC_HIDDEN	3


#ifdef	CLASSY

#include "class(y).ch"

#command _CY <*rest*>	=>	<rest>

#command _CY METHOD <name>( [<p,...>] ) [<*rest*>]			;
      => METHOD <name>		// <rest>

#command METHOD <name>( [<p,...>] )					;
      => METHOD <name>

#command DECL_METHOD <name>( [<p,...>] )				;
      => DECL_METHOD <name>

#command _CY ASSIGN <name>( <p> )					;
      => METHOD <name>

#command _CY ASSIGN <name>( <p> ) INLINE <*exp*>			;
      => METHOD FUNCTION <name>( <p> )   ; RETURN <exp>

#command _TC <*rest*>	=>

#else	// CLASSY

#command _CY <*rest*>	=>
#command _TC <*rest*>	=>	<rest>

#endif	// CLASSY


// Class Definitions:

#ifdef	CLASSY

#command CLASS <name>			=>	CREATE CLASS <name>

#command CLASS <name> INHERIT <sup>					;
      => CREATE CLASS <name> INHERIT <sup>

#else	// CLASSY

#command CLASS <name>							;
	 [<notobj: NOINHERIT FROM OBJECT>]				;
									;
      => FUNCTION <name>()						;
       ; STATIC	_t_classh						;
       ; LOCAL	self							;
									;
       ; IF _t_classh == NIL						;
       ;	_t_NewClass( self := UPPER(#<name>)			;
			     [, , <.notobj.> ] )

#command CLASS <name>							;
	 <from:INHERIT FROM,INHERIT,FROM> <superclass>			;
	 [<notobj: NOINHERIT FROM OBJECT>]				;
									;
      => FUNCTION <name>()						;
       ; STATIC	_t_classh						;
       ; LOCAL	self							;
									;
       ; IF _t_classh == NIL						;
       ;	_t_NewClass( self := UPPER(#<name>),			;
			     <superclass>()				;
			     [, <.notobj.> ] )

#command CLASS <name>							;
	 <from:INHERIT FROM,INHERIT,FROM> <sc1>, <sc2> [, <scN> ]	;
	 [<notobj: NOINHERIT FROM OBJECT>]				;
									;
      => FUNCTION <name>()						;
       ; STATIC	_t_classh						;
       ; LOCAL	self							;
									;
       ; IF _t_classh == NIL						;
       ;	_t_NewClass( self := UPPER(#<name>),			;
			     <sc1>(),					;
			     { <sc2>()  [, <scN>() ] }			;
			     [, <.notobj.> ] )

#endif	// CLASSY


// Strong typing (internal use):

#define	_TC_ANY		0
#define	_TC_USUAL	0
#define	_TC_INT		10	// signed, 32-bit integer
//	_TC_LONGINT	10
//	_TC_DWORD	10	// sorry, this is the best you get
#define	_TC_NUMERIC	20
#define	_TC_STRING	30
#define	_TC_LOGIC	40
#define	_TC_BLOCK	50
//	_TC_CODEBLOCK	50
#define	_TC_DATE	60
#define	_TC_ARRAY	70
#define	_TC_OBJECT	80
#define	_TC_SHORT	90	// signed, 16-bit integer
#define	_TC_WORD	100	// unsigned, 16-bit integer


// Instance Variables:

#ifdef	CLASSY

#command _VAR_ <type> <iv>						;
      => VAR <iv>

#command _VAR_ <type> <iv>, <ivN,...>					;
      => VAR <iv>							;
       ; _VAR_ <type> <ivN>

#command _VAR_ <scope> <iv,...> AS <x:USUAL,ANY>			;
      => _VAR_ <scope>+_TC_ANY <iv>

#command _VAR_ <scope> <iv,...> AS <i:INT,LONG,LONGINT>			;
      => _VAR_ <scope>+_TC_INT <iv>

#command _VAR_ <scope> <iv,...> AS <i:SHORT,SHORTINT>			;
      => _VAR_ <scope>+_TC_SHORT <iv>

#command _VAR_ <scope> <iv,...> AS WORD					;
      => _VAR_ <scope>+_TC_WORD <iv>

#command _VAR_ <scope> <iv,...> AS NUMERIC				;
      => _VAR_ <scope>+_TC_NUMERIC <iv>

#command _VAR_ <scope> <iv,...> AS STRING				;
      => _VAR_ <scope>+_TC_STRING <iv>

#command _VAR_ <scope> <iv,...> AS LOGIC				;
      => _VAR_ <scope>+_TC_LOGIC <iv>

#command _VAR_ <scope> <iv,...> AS <b:BLOCK,CODEBLOCK>			;
      => _VAR_ <scope>+_TC_BLOCK <iv>

#command _VAR_ <scope> <iv,...> AS DATE					;
      => _VAR_ <scope>+_TC_DATE <iv>

#command _VAR_ <scope> <iv,...> AS ARRAY				;
      => _VAR_ <scope>+_TC_ARRAY <iv>

#command _VAR_ <scope> <iv,...> AS OBJECT				;
      => _VAR_ <scope>+_TC_OBJECT <iv>

#command _VAR_ <scope> [<clauses,...>] READONLY				;
      => _VAR_ <scope>+_TC_READONLY [<clauses>]

#else	// CLASSY

#command _VAR_ <type> <iv>						;
      => _tc_IvName(UPPER(#<iv>), <type>)

#command _VAR_ <type> <iv>, <ivN,...>					;
      => _tc_IvName(UPPER(#<iv>), <type>)				;
       ; _VAR_ <type> <ivN>

#command _VAR_ <scope> <iv,...> AS <x:USUAL,ANY>			;
      => _VAR_ <scope>+_TC_ANY <iv>

#command _VAR_ <scope> <iv,...> AS <i:INT,LONG,LONGINT>			;
      => _VAR_ <scope>+_TC_INT <iv>

#command _VAR_ <scope> <iv,...> AS <i:SHORT,SHORTINT>			;
      => _VAR_ <scope>+_TC_SHORT <iv>

#command _VAR_ <scope> <iv,...> AS WORD					;
      => _VAR_ <scope>+_TC_WORD <iv>

#command _VAR_ <scope> <iv,...> AS NUMERIC				;
      => _VAR_ <scope>+_TC_NUMERIC <iv>

#command _VAR_ <scope> <iv,...> AS STRING				;
      => _VAR_ <scope>+_TC_STRING <iv>

#command _VAR_ <scope> <iv,...> AS LOGIC				;
      => _VAR_ <scope>+_TC_LOGIC <iv>

#command _VAR_ <scope> <iv,...> AS <b:BLOCK,CODEBLOCK>			;
      => _VAR_ <scope>+_TC_BLOCK <iv>

#command _VAR_ <scope> <iv,...> AS DATE					;
      => _VAR_ <scope>+_TC_DATE <iv>

#command _VAR_ <scope> <iv,...> AS ARRAY				;
      => _VAR_ <scope>+_TC_ARRAY <iv>

#command _VAR_ <scope> <iv,...> AS OBJECT				;
      => _VAR_ <scope>+_TC_OBJECT <iv>

#command _VAR_ <scope> [<clauses,...>] READONLY				;
      => _VAR_ <scope>+_TC_READONLY [<clauses>]

#endif	// CLASSY


// Class Variables:

#ifdef	CLASSY

/*
#command CLASSVAR <name> ACCESS <bAccess> ASSIGN <bAssign>		;
      => MESSAGE <name> ACCESS <bAccess> ASSIGN <bAssign>

#command CLASSVAR <name> ASSIGN <bAssign> ACCESS <bAccess>		;
      => CLASSVAR <name> ACCESS <bAccess> ASSIGN <bAssign>
*/

#xcommand CLASSVAR <name>						;
      => CLASS VAR <name>

#command CLASSVAR <name> STATIC <svar>					;
      => METHOD FUNCTION <name> ; RETURN <svar>

#command _CY CLASSVAR <name> STATIC <svar>				;
      => METHOD <name>

#else	// CLASSY

#command CLASSVAR <name> ACCESS <bAccess> ASSIGN <bAssign>		;
      => MESSAGE <name> ACCESS <bAccess> ASSIGN <bAssign>

#command CLASSVAR <name> ASSIGN <bAssign> ACCESS <bAccess>		;
      => CLASSVAR <name> ACCESS <bAccess> ASSIGN <bAssign>

#xcommand CLASSVAR <name>						;
      => CLASSVAR <name> ACCESS {|| <name>} ASSIGN {|SELF, _1| <name> := _1}

#command CLASSVAR <name> STATIC <svar>					;
      => CLASSVAR <name> ACCESS {|| <svar>} ASSIGN {|SELF, _1| <svar> := _1}

#endif	// CLASSY


// Messages:

#ifdef	CLASSY

#command MESSAGE <name> DEFERRED					;
      => _tc_DfrMsg(UPPER(#<name>))

#command MESSAGE <name>( [<any,...>] ) DEFERRED				;
      => _tc_DfrMsg(UPPER(#<name>))

#command MESSAGE <name> BLOCK <b>					;
      => _tc_Method(UPPER(#<name>), <b>)

#command MESSAGE block BLOCK <b>					;
      => _tc_Method("BLOCK", <b>)

#command MESSAGE <name>([<any,...>]) BLOCK <b>				;
      => MESSAGE <name> BLOCK <b>

#command MESSAGE <name> METHOD <meth>					;
      => MESSAGE <name> METHOD <meth>()

#command MESSAGE <name> METHOD <meth>([<p1>] [, <pN>])			;
      => MESSAGE <name> BLOCK {|SELF [, <p1>] [, <pN>]| <meth>( SELF [, @<p1>] [, @<pN>]) }

#command MESSAGE <name>							;
      => MESSAGE <name> METHOD <name>()

#command MESSAGE <name>([<p,...>])					;
      => MESSAGE <name> METHOD <name>(<p>)

#command MESSAGE <name>([<p,...>]) METHOD <meth>			;
      => MESSAGE <name> METHOD <meth>(<p>)

#command MESSAGE <name>([<any,...>]) METHOD <meth>([<p,...>])		;
      => MESSAGE <name> METHOD <meth>(<p>)

#command MESSAGE <name>([<p,...>]) METHOD <meth>()			;
      => MESSAGE <name> METHOD <meth>(<p>)

#command MESSAGE <name> INLINE <*exp*>					;
      => MESSAGE <name>() INLINE <exp>

//#command MESSAGE <name>([<p1>] [, <pN>]) INLINE <*exp*>			;
//      => MESSAGE <name> BLOCK {|SELF [, <p1>] [, <pN>]| <exp>}

#command MESSAGE <name>([<p1>] [, <pN>]) INLINE [LOCAL <v>,] <*exp*>	;
      => MESSAGE <name> BLOCK {|SELF [, <p1>] [, <pN>] [, <v>]| <exp>}

/*
#command MESSAGE <name> ACCESS <bAccess> ASSIGN <bAssign>		;
      => _tc_Method(UPPER(#<name>), <bAccess>)				;
       ; _tc_Method("_"+UPPER(#<name>), <bAssign>)

#command MESSAGE <name> ASSIGN <bAssign> ACCESS <bAccess>		;
      => MESSAGE <name> ACCESS <bAccess> ASSIGN <bAssign>

#command MESSAGE <name> DELEGATE [TO] <ivar>				;
      => MESSAGE <name>() DELEGATE TO <ivar>

#command MESSAGE <name>([<p1>] [, <pN>]) DELEGATE [TO] <ivar>		;
      => MESSAGE <name> BLOCK {|SELF [, <p1>] [, <pN>]|			;
                               SELF:<ivar>:<name>([@<p1>] [, @<pN>])}
*/

#else	// CLASSY

#command MESSAGE <name> DEFERRED					;
      => _tc_DfrMsg(UPPER(#<name>))

#command MESSAGE <name>( [<any,...>] ) DEFERRED				;
      => _tc_DfrMsg(UPPER(#<name>))

#command MESSAGE <name> BLOCK <b>					;
      => _tc_Method(UPPER(#<name>), <b>)

#command MESSAGE block BLOCK <b>					;
      => _tc_Method("BLOCK", <b>)

#command MESSAGE <name>([<any,...>]) BLOCK <b>				;
      => MESSAGE <name> BLOCK <b>

#command MESSAGE <name> METHOD <meth>					;
      => MESSAGE <name> METHOD <meth>()

#command MESSAGE <name> METHOD <meth>([<p1>] [, <pN>])			;
      => MESSAGE <name> BLOCK {|SELF [, <p1>] [, <pN>]| <meth>( SELF [, @<p1>] [, @<pN>]) }

#command MESSAGE <name>							;
      => MESSAGE <name> METHOD <name>()

#command MESSAGE <name>([<p,...>])					;
      => MESSAGE <name> METHOD <name>(<p>)

#command MESSAGE <name>([<p,...>]) METHOD <meth>			;
      => MESSAGE <name> METHOD <meth>(<p>)

#command MESSAGE <name>([<any,...>]) METHOD <meth>([<p,...>])		;
      => MESSAGE <name> METHOD <meth>(<p>)

#command MESSAGE <name>([<p,...>]) METHOD <meth>()			;
      => MESSAGE <name> METHOD <meth>(<p>)

#command MESSAGE <name> INLINE <*exp*>					;
      => MESSAGE <name>() INLINE <exp>

//#command MESSAGE <name>([<p1>] [, <pN>]) INLINE <*exp*>			;
//      => MESSAGE <name> BLOCK {|SELF [, <p1>] [, <pN>]| <exp>}

#command MESSAGE <name>([<p1>] [, <pN>]) INLINE [LOCAL <v>,] <*exp*>	;
      => MESSAGE <name> BLOCK {|SELF [, <p1>] [, <pN>] [, <v>]| <exp>}

#command MESSAGE <name> ACCESS <bAccess> ASSIGN <bAssign>		;
      => _tc_Method(UPPER(#<name>), <bAccess>)				;
       ; _tc_Method("_"+UPPER(#<name>), <bAssign>)

#command MESSAGE <name> ASSIGN <bAssign> ACCESS <bAccess>		;
      => MESSAGE <name> ACCESS <bAccess> ASSIGN <bAssign>

#command MESSAGE <name> DELEGATE [TO] <ivar>				;
      => MESSAGE <name>() DELEGATE TO <ivar>

#command MESSAGE <name>([<p1>] [, <pN>]) DELEGATE [TO] <ivar>		;
      => MESSAGE <name> BLOCK {|SELF [, <p1>] [, <pN>]|			;
                               SELF:<ivar>:<name>([@<p1>] [, @<pN>])}

#endif	// CLASSY


// Useful and miscellaneous:

#translate  ::		=>	SELF:

#ifdef	TC_UNIFORMREFERENCE

// Please don't want to leave out the parens for a msg in Super!

#xtranslate SUPER:<msg>							;
         => EVAL( _tc_SuperMsg(SELF, UPPER(#<msg>)), SELF )

// intended for multiple inheritance only
#xtranslate SUPER( <cSup> ):<msg>					;
         => EVAL( _tc_SuperC(UPPER(<(cSup)>), UPPER(#<msg>)), SELF )

#xtranslate QSUPER( <svar> ):<msg>					;
         => EVAL( IIF(<svar> == NIL,					;
                      <svar> := _tc_SuperMsg(SELF, UPPER(#<msg>)),	;
                      <svar>), SELF )

#endif	// TC_UNIFORMREFERENCE

#ifdef	CLASSY

// This one could be an ASSIGN or a comparison (e.g. IF SUPER:xxx = "yyy")
// here's ASSIGN:
//#xtranslate SUPER:<msg> = <v>						;
//         => EVAL( _tc_SuperA(SELF, "_"+UPPER(#<msg>)), SELF, <v>)

/* TBD:
#xtranslate SUPER:<msg> := <v>						;
         => EVAL( _tc_SuperA(SELF, "_"+UPPER(#<msg>)), SELF, <v>)
*/

/* TBD:
// intended for multiple inheritance only
#xtranslate SUPER( <cSup> ):<msg>( [<p,...>] )				;
         => EVAL( _tc_SuperC(UPPER(<(cSup)>), UPPER(#<msg>)), SELF [, <p>] )
*/

/* TBD:
#xtranslate QSUPER( <svar> ):<msg>( [<p,...>] )				;
         => EVAL( IIF(<svar> == NIL,					;
                      <svar> := _tc_SuperMsg(SELF, UPPER(#<msg>)),	;
                      <svar>),						;
                  SELF [, <p>] )
*/

#xtranslate = SUPER:	=>	= SELF:super:

#xtranslate := SUPER:	=>	:= SELF:super:

#xtranslate ( SUPER:	=>	( SELF:super:

#xtranslate , SUPER:	=>	, SELF:super:

#xtranslate RETURN SUPER:	=>	RETURN SELF:super:

#xtranslate { SUPER:		=>	{ SELF:super:

#xtranslate {|[<p,...>]| SUPER:	=> {|[<p>]| SELF:super:

#ifdef	TC_MORE

#xtranslate == SUPER:	=>	== SELF:super:

#xtranslate += SUPER:	=>	+= SELF:super:

#xtranslate -= SUPER:	=>	-= SELF:super:

//etc.

#endif	// TC_MORE

#else	// CLASSY

// This one could be an ASSIGN or a comparison (e.g. IF SUPER:xxx = "yyy")
// here's ASSIGN:
//#xtranslate SUPER:<msg> = <v>						;
//         => EVAL( _tc_SuperA(SELF, "_"+UPPER(#<msg>)), SELF, <v>)

#xtranslate SUPER:<msg> := <v>						;
         => EVAL( _tc_SuperA(SELF, "_"+UPPER(#<msg>)), SELF, <v>)

#xtranslate SUPER:<msg>( [<p,...>] )					;
         => EVAL( _tc_SuperMsg(SELF, UPPER(#<msg>)), SELF [, <p>] )

// intended for multiple inheritance only
#xtranslate SUPER( <cSup> ):<msg>( [<p,...>] )				;
         => EVAL( _tc_SuperC(UPPER(<(cSup)>), UPPER(#<msg>)), SELF [, <p>] )

#xtranslate QSUPER( <svar> ):<msg>( [<p,...>] )				;
         => EVAL( IIF(<svar> == NIL,					;
                      <svar> := _tc_SuperMsg(SELF, UPPER(#<msg>)),	;
                      <svar>),						;
                  SELF [, <p>] )

#endif	// CLASSY


// Support MyClass{ ... } syntax:

// This doesn't work
//#xtranslate <name>\{ [<p,...>] }	=> <name>():Init( <p> )

#xtranslate := ():Init( [<p,...>] )					;
         => := ({ <p> })

#xtranslate = <name>{ [<p,...>] }					;
         => = <name>():Init( <p> )

#xtranslate := <name>{ [<p,...>] }					;
         => := <name>():Init( <p> )

#xtranslate ( <name>{ [<p,...>] }					;
         => ( <name>():Init( <p> )

#xtranslate , <name>{ [<p,...>] }					;
         => , <name>():Init( <p> )

#xcommand RETURN <name>{ [<p,...>] }					;
       => RETURN <name>():Init( <p> )

#xtranslate { <name>{ [<p,...>] }					;
         => { <name>():Init( [<p>] )

#xtranslate {|[<p,...>]| { [<e,...>] }					;
         => {|[<p>]| ({ <e> })

#ifdef	TC_MORE

#xtranslate == <name>{ [<p,...>] }					;
         => == <name>():Init( [<p>] )

#xtranslate += <name>{ [<p,...>] }					;
         => += <name>():Init( [<p>] )

#xtranslate -= <name>{ [<p,...>] }					;
         => -= <name>():Init( [<p>] )

//etc.

#endif	// TC_MORE


#xtranslate tc_Send( <o>, <msg> [, <p,...>] )				;
         => EVAL( _tc_SelfO(<o>, UPPER(<msg>)), SELF [, <p>] )

#xtranslate tc_Send( <o>, <msg>, SELF [, <p,...>] )			;
         => EVAL( _tc_SelfO(<o>, UPPER(<msg>)), SELF [, <p>] )


// TBD:
//	:SUPER( ... )
//	:QSUPER( ... )
//	:CLASS:				// may be OK
//	CLASSVAR <name> AS <type>
//	INIT CLASS
//	more VO compatibility
//	more Class(y) compatibility


// Compatibility:


#ifdef	TC_CLIPPERCLASSES

#xtranslate Get():	=>	t_Get():
#xtranslate Error():	=>	t_Error():
#xtranslate TBrowse():	=>	t_TBrowse():
#xtranslate TBColumn():	=>	t_TBColumn():

#endif	// TC_CLIPPERCLASSES


// Instance variables:

#ifdef	CLASSY

// Use this, mainly
#command PROTECT <*rest*>						;
      => PROTECTED:							;
       ; _VAR_ _TC_PROTECTED <rest>

#command HIDDEN <*rest*>						;
      => HIDDEN:							;
       ; _VAR_ _TC_IV_HIDDEN <rest>

#command EXPORT <*rest*>						;
      => EXPORTED:							;
       ; _VAR_ _TC_PUBLIC <rest>

// You don't want to use this!
#command INSTANCE <*rest*>						;
      => EXPORTED:							;
       ; _VAR_ _TC_PUBLIC <rest>

#else	// CLASSY

// Use this, mainly
#command PROTECTED <*rest*>						;
      => _VAR_ _TC_PROTECTED <rest>

#command HIDDEN <*rest*>						;
      => _VAR_ _TC_IV_HIDDEN <rest>

#command EXPORTED <*rest*>						;
      => _VAR_ _TC_PUBLIC <rest>

// You don't want to use this!
#command INSTANCE <*rest*>						;
      => _VAR_ _TC_PUBLIC <rest>

#endif	// CLASSY


#ifdef	CLASSY

#command ACCESS <name>							;
      => METHOD <name>

#command ACCESS <name> DEFERRED						;
      => MESSAGE <name> DEFERRED

//#command ACCESS <name> INLINE [LOCAL <v>,] <*exp*>			;
//      => MESSAGE <name>(<v>) INLINE <exp>

#command ACCESS <name> INLINE <*exp*>					;
      => METHOD FUNCTION <name>() ; RETURN <exp>

#command ASSIGN <name>(<p>)						;
      => METHOD <name>(<p>)

// This would need token pasting (not in Clipper's pre-processor):
//#command ASSIGN <name> INLINE <*exp*>					;
//      => MESSAGE _<name> INLINE <exp>

#command ASSIGN <name>(<p>) INLINE <*exp*>				;
      => METHOD FUNCTION <name>(<p>) ; RETURN <exp>

// Method declaration:

#command METHOD <name>([<p,...>]) INLINE <*exp*>			;
      => METHOD FUNCTION <name>(<p>) ; RETURN <exp>

#else	// CLASSY

#command ACCESS <name>							;
      => METHOD <name>

#command ACCESS <name> DEFERRED						;
      => MESSAGE <name> DEFERRED

//#command ACCESS <name> INLINE [LOCAL <v>,] <*exp*>			;
//      => MESSAGE <name>(<v>) INLINE <exp>

#command ACCESS <name> INLINE <*exp*>					;
      => MESSAGE <name>() INLINE <exp>

#command ASSIGN <name>(<p>)						;
      => ASSIGN <name>(<p>) INLINE <name>(SELF, <p>)

// This would need token pasting (not in Clipper's pre-processor):
//#command ASSIGN <name> INLINE <*exp*>					;
//      => MESSAGE _<name> INLINE <exp>

#command ASSIGN <name>(<p>) INLINE <*exp*>				;
      => _tc_Method("_"+UPPER(#<name>), {|SELF, <p>| <exp>})

// Method declaration:
#command METHOD <*rest*>						;
      => MESSAGE <rest>

#endif	// CLASSY


// Method definition:

#ifdef	CLASSY

#command ACCESS <name> CLASS <class>					;
      => METHOD <name> CLASS <class>					;

#command ASSIGN <name> CLASS <class>					;
      => METHOD <name> CLASS <class>					;

#command METHOD <name> CLASS <class>					;
      => METHOD FUNCTION <name>			// CLASS <class>

#command METHOD <name>( [<p,...>] ) CLASS <class>			;
      => METHOD FUNCTION <name>( <p> )		// CLASS <class>

#else	// CLASSY

#command ACCESS <name> CLASS <class>					;
      => METHOD <name> CLASS <class>					;

#command ASSIGN <name> CLASS <class>					;
      => METHOD <name> CLASS <class>					;

#command METHOD <name> CLASS <class>					;
      => STATIC FUNCTION <name>( SELF )			// CLASS <class>

#command METHOD <name>( [<p,...>] ) CLASS <class>			;
      => STATIC FUNCTION <name>( SELF [, <p>] )		// CLASS <class>

#endif	// CLASSY


#ifdef	TBD_TBD  // not done yet

// Renaming - but you're making yourself more work in VO if you use this!
//
// Intended for use with multiple inheritance to remove any conflicts.
// Only allowed for EXPORTs / PUBLICs.

#xcommand RENAME VAR <name> TO <new>					;
       => _tc_RenIVar(UPPER(#<name>), UPPER(#<new>))

#xcommand RENAME MESSAGE <name> TO <new>				;
       => _tc_RenMsg(UPPER(#<name>), UPPER(#<new>))

#endif	// TBD_TBD


#include "vo.ch"


#ifdef	CLASSY

#xcommand SUPER:<*rest*>  =>	SELF:super:<rest>

#else	// CLASSY

#xtranslate :CLASS:	=>	:	// and hope!

#endif	// CLASSY


#ifdef	CPLUSPLUS

#command METHOD <class>::<meth>( [<p,...>] )				;
      => STATIC FUNCTION <meth>( SELF [, <p>] )		// CLASS <class>

#endif	// CPLUSPLUS


#ifdef	SUPERCLASS

#command PUBLIC		=>	_tc_IvType( _TC_PUBLIC )
#command PROTECTED	=>	_tc_IvType( _TC_PROTECTED )
#command READ ONLY	=>	_tc_IvType( _TC_READONLY )
#command RESTRICTED	=>	_tc_IvType( _TC_IV_HIDDEN )

#command PUBLIC <m: MESSAGES, METHODS>       => _tc_MsgType( _TC_PUBLIC )
#command PROTECTED <m: MESSAGES, METHODS>    => _tc_MsgType( _TC_PROTECTED )
#command READ ONLY <m: MESSAGES, METHODS>    => _tc_MsgType( _TC_READONLY )
#command RESTRICTED <m: MESSAGES, METHODS>   => _tc_MsgType( _TC_HIDDEN )

#command EXPORTED		=>	PUBLIC
#command EXPORTED METHODS	=>	PUBLIC METHODS

#command CALCULATED <name> [ACCESS <bAccess>] [ASSIGN <bAssign>]	;
      => MESSAGE <name> [ACCESS <bAccess>] [ASSIGN <bAssign>]

#command DELEGATE <name> TO <ivar>					;
      => MESSAGE <name>() DELEGATE TO <ivar>

#command DELEGATE <name>([<p1>] [, <pN>]) TO <ivar>			;
      => MESSAGE <name>([<p1>] [, <pN>]) DELEGATE TO <ivar>

#endif	// SUPERCLASS


#ifdef	CLASSY
#else

#command INIT CLASS							;
      =>	_t_classh := _t_ClassDone()				;
       ;	SELF := __ClassIns( _t_classh )
		// Your class initialisation must come next
		// (followed by ENDCLASS and with no more ivars or messages)

#command ENDCLASS							;
      =>	_t_classh := _t_ClassDone()				;
       ; END IF								;
       ; RETURN __ClassIns( _t_classh )		// Not here... :Init()

#command END CLASS							;
      => ENDCLASS							;

#endif	// CLASSY

#endif	// TOPCLASS_CH
