////////////////////////////
//
//	Clip-4-Win OO brush sample
//
//	Copyright (C) 1992-1995 Skelton Software, Kendal Cottage, Hillam, Leeds, UK.
//	All Rights Reserved.
//
//	Compile and link:  c4wbuild brush
//
////////////////////////////

#include "windows.ch"
#include "commands.ch"

#define	W_LEFT		1
#define	W_TOP		2
#define	W_RIGHT		3
#define	W_BOTTOM	4

#define C_RED        RGB(255,0,0)
#define C_BLUE       RGB(0,0,255)
#define C_GREEN      RGB(0,255,0)
#define C_MAGENTA    RGB(255,0,255)
#define C_BLACK      RGB(0,0,0)

function main()
local	oApp, oWnd

oApp = App{"Clip-4-Win Demo: Using Brushes"}
oApp:oWnd:MenuSetup()

oApp:Start()

oApp:Quit()
return nil


CLASS App INHERIT WMDIApp
	METHOD Create(cText)	INLINE ::oWnd := MainWindow{self, cText}
ENDCLASS


CLASS MainWindow INHERIT WMDIFrame
	METHOD BrushBg()						;
	       INLINE local   nColor,					;
	              nColor := ChooseColor(),				;
	              ::CreateChild(Background{::Client, "Using a bg brush", nColor})

	METHOD BrushBar()						;
	       INLINE ::CreateChild(BarChart{::Client, "Bar Chart"})

	METHOD BrushPie()						;
	       INLINE ::CreateChild(PieChart{::Client, "Pie Chart"})

	METHOD BrushRect()						;
	       INLINE local   nColor,					;
	              nColor := ChooseColor(),				;
	              ::CreateChild(Rect{::Client, "FillRect( ) function", nColor})

	METHOD FileExit()	INLINE ::App:Quit()
	METHOD HelpAbout()	INLINE ::MsgBox("Put your Help|About here!")
	METHOD MenuSetup()
ENDCLASS


CLASS Background INHERIT WMDIChild
	PROTECT ClassName AS STRING	// to create multiple names
	PROTECT nColor	AS LONG
	METHOD Init(oParent, cTitle, nColor)				;
	       INLINE ::nColor := nColor,				;
	              ::ClassName := alltrim(str(nColor)),		;
	              super:Init(oParent, cTitle)
	METHOD RegisterClass(hInst, hIcon, hCursor, hBrush)		;
	       INLINE hBrush := CreateHatchBrush(HS_BDIAGONAL, ::nColor),;
	              super:RegisterClass(hInst, hIcon, hCursor, hBrush)
ENDCLASS


CLASS BarChart INHERIT WMDIChild
	MESSAGE Paint()		METHOD BarPaint()
ENDCLASS


METHOD BarPaint() CLASS BarChart
local hDC := ::hDC, aRect, hBrush, hOldbrush, nWidth
   hBrush := CreateHatchBrush(HS_BDIAGONAL, C_RED)
   aRect := GetClientRect(::hWnd)
   nWidth := aRect[W_RIGHT] / 6
   hOldbrush := SelectObject(hDC, hBrush)
   TextOut(hDC, nWidth + 10, aRect[W_BOTTOM] * .15 - 20, "U.S.A.")
   Rectangle(hDC, nWidth, aRect[W_BOTTOM] * .15, ;
		  nWidth * 2, aRect[W_BOTTOM])
   hBrush := CreateHatchBrush(HS_VERTICAL, C_MAGENTA)
   DeleteObject( SelectObject(hDC, hBrush) )
   TextOut(hDC, nWidth * 2 + 10, aRect[W_BOTTOM] * .4 - 20, "England")
   Rectangle(hDC, nWidth * 2, aRect[W_BOTTOM] * .4, ;
		  nWidth * 3, aRect[W_BOTTOM])
   hBrush := CreateHatchBrush(HS_FDIAGONAL, C_BLUE)
   DeleteObject( SelectObject(hDC, hBrush) )
   TextOut(hDC, nWidth * 3 + 10, aRect[W_BOTTOM] * .8 - 20, "Australia")
   Rectangle(hDC, nWidth * 3, aRect[W_BOTTOM] * .8, ;
		  nWidth * 4, aRect[W_BOTTOM])
   hBrush := CreateHatchBrush(HS_CROSS, C_GREEN)
   DeleteObject( SelectObject(hDC, hBrush) )
   TextOut(hDC, nWidth * 4 + 10, aRect[W_BOTTOM] * .55 - 20, "Germany")
   Rectangle(hDC, nWidth * 4, aRect[W_BOTTOM] * .55, ;
		  nWidth * 5, aRect[W_BOTTOM])
   DeleteObject( SelectObject(hDC, hOldbrush) )
return nil


CLASS PieChart INHERIT WMDIChild
	PROTECT nColor	AS LONG
	METHOD Init(oParent, cTitle, nColor)				;
	       INLINE ::nColor := nColor,				;
	              super:Init(oParent, cTitle)
	MESSAGE Paint()		METHOD PiePaint()
ENDCLASS


METHOD PiePaint() CLASS PieChart
local hDC := ::hDC, aRect, hBrush, hOldbrush
   hBrush := CreateSolidBrush(C_RED)
   aRect := GetClientRect(::hWnd)
   hOldbrush := SelectObject(hDC, hBrush)
   Pie(hDC, 0, 0, aRect[W_RIGHT], aRect[W_BOTTOM], ;
	    aRect[W_RIGHT] / 2, 0, aRect[W_RIGHT], aRect[W_BOTTOM])
   hBrush := CreateSolidBrush(C_MAGENTA)
   DeleteObject( SelectObject(hDC, hBrush) )

   Pie(hDC, 0, 0, aRect[W_RIGHT], aRect[W_BOTTOM], aRect[W_RIGHT], ;
	    aRect[W_BOTTOM], aRect[W_RIGHT] * .75, aRect[W_BOTTOM] * .25)
   hBrush := CreateSolidBrush(C_BLUE)
   DeleteObject( SelectObject(hDC, hBrush) )
   Pie(hDC, 0, 0, aRect[W_RIGHT], aRect[W_BOTTOM], aRect[W_RIGHT], ;
	    aRect[W_BOTTOM] / 2, aRect[W_RIGHT] * .75, aRect[W_BOTTOM] * .25)
   hBrush := CreateSolidBrush(C_GREEN)
   DeleteObject( SelectObject(hDC, hBrush) )
   Pie(hDC, 0, 0, aRect[W_RIGHT], aRect[W_BOTTOM], ;
       aRect[W_RIGHT] * .75, aRect[W_BOTTOM] * .25, aRect[W_RIGHT] / 2, 0)
   DeleteObject( SelectObject(hDC, hOldbrush) )
return nil


CLASS Rect INHERIT WMDIChild
	PROTECT nColor	AS LONG
	METHOD Init(oParent, cTitle, nColor)				;
	       INLINE ::nColor := nColor,				;
	              super:Init(oParent, cTitle)
	METHOD Paint()							;
	       INLINE local	hBrush,					;
	              local	a,					;
	              a := GetClientRect(::hWnd),			;
	              hBrush := CreateSolidBrush(::nColor),		;
	              FillRect(::hDC, 20, 20, a[3] - 20, a[4] - 20, hBrush),;
	              DeleteObject(hBrush)
ENDCLASS


METHOD MenuSetup() CLASS MainWindow
MENU IN self
    POPUP "&File"
        MENUITEM "E&xit"  COMMAND FileExit
    ENDPOPUP
    POPUP "&Brush"
        MENUITEM "Back&ground Brush"  COMMAND BrushBg
        MENUITEM "&Bar Chart"         COMMAND BrushBar
        MENUITEM "&Pie Chart"         COMMAND BrushPie
        MENUITEM "&Rectangle"         COMMAND BrushRect
    ENDPOPUP
    MDIWINDOWPOPUP
    POPUP "&Help"
        MENUITEM "&About"  COMMAND HelpAbout
    ENDPOPUP
ENDMENU
return self
