////////////////////////////
//
//	Clip-4-Win DBF generator
//
//	Copyright (C) 1994 Skelton Software, Kendal Cottage, Hillam, Leeds, UK.
//	All Rights Reserved.
//
//	Compile and link:	c4wbuild dbfgen
//
////////////////////////////

#include "dbstruct.ch"
#include "topclass.ch"
#define	WIN_WANT_ALL
#include "windows.ch"
#include "commands.ch"
#include "dbfgen.ch"

#define	TAB		chr(9)
#define	ARemove(a, i)	ASize(ADel(a, i), len(a) - 1)

function main()
local	oApp

oApp = MyApp{"Clip-4-Win DBF Generator"}
oApp:oWnd:MenuSetup()

oApp:Start()

return nil


CLASS MyApp INHERIT WApp
	METHOD Create(cText)	INLINE ::oWnd := MainWindow{self, cText}
ENDCLASS


CLASS MainWindow INHERIT WSDIWindow
	METHOD DBFNew()		INLINE DBFDialog{self, "dbfgen"}
	METHOD Exit()		INLINE ::oApp:Quit(), self
	METHOD Init(oApp, cText)					;
	       INLINE super:Init(oApp, cText),				;
	              ::StatusBar := WStatusBar{self},			;
	              self
	METHOD MenuSetup()
ENDCLASS


CLASS DBFDialog INHERIT WDialog

	PROTECT aFlds AS ARRAY
	PROTECT aStruct AS ARRAY

	EXPORT cType, cLen, cDec AS STRING

	// some Edit controls
	PROTECT	oDBFName, oFldName, oLen, oDec AS OBJECT

	PROTECT oFields AS OBJECT

	METHOD Type(cType)						;
	       INLINE ::cType := cType,					;
	              ::cLen  := {"20", "10", "8", "100", "1"}		;
	                         [at(cType, "CNDML")],			;
	              ::oLen:UpdateBuffer(),				;
	              ::cDec  := "0",					;
	              ::oDec:UpdateBuffer(),				;
	              1

	// buttons - Cancel() can default
	METHOD CreateDBF()
	METHOD OpenDBF()
	METHOD AddFld()
	METHOD DelFld()

	METHOD OnInitDialog()
ENDCLASS


METHOD OnInitDialog() CLASS DBFDialog
local	oChar
::aFlds   = {}
::aStruct = {}
::cLen    = "20"
::cDec    = "0"

@ Id IDD_DBFNAME  Obj ::oDBFName  Edit
@ Id IDD_FLDNAME  Obj ::oFldName  Edit

@ Id IDD_LEN  Obj ::oLen  Get ::cLen  Picture "999"  Valid val(::cLen) != 0

@ Id IDD_DEC  Obj ::oDec  Get ::cDec  Picture "9"

::Type("C")

@ Id IDD_FLDLIST  Obj ::oFields   ListBox  Items ::aFlds

// buttons: (Cancel can default)
@ Id IDD_GEN  PushButton CreateDBF
@ Id IDD_OPEN PushButton OpenDBF
@ Id IDD_ADD  PushButton AddFld
@ Id IDD_DEL  PushButton DelFld

// radio buttons
@ Id IDD_CHAR  RadioButton  Action ::Type("C")  Obj oChar
oChar:Checked := .t.
@ Id IDD_NUM   RadioButton  Action ::Type("N")
@ Id IDD_DATE  RadioButton  Action ::Type("D")
@ Id IDD_MEMO  RadioButton  Action ::Type("M")
@ Id IDD_LOGIC RadioButton  Action ::Type("L")

return super:OnInitDialog()


METHOD AddFld() CLASS DBFDialog
local	cFld, i
if ::Valid
	cFld = strtran(left(alltrim(::oFldName:Text), 10), " ", "_")
	if empty(cFld)
		::MsgBox("Please specify the Field Name", "DBFGen")
		return 1
	endif
	if empty(::cLen)
		::MsgBox("Please specify a valid Length", "DBFGen")
		return 1
	endif
	if (i := ascan(::aStruct, {|a| a[DBS_NAME] == cFld})) != 0
		if ::MsgBox("Replace field?", "DBFGen", MB_YESNO) != IDYES
			return 1
		else
			ARemove(::aStruct, i)
			::oFields:Delete(i)
		endif
	endif
	aadd(::aStruct, {cFld, ::cType, val(::cLen), val(::cDec)})
	::oFields:Add(cFld + TAB + ::cType + TAB + ::cLen + TAB + ::cDec)
	::oFields:GoBottom()
else
	MessageBeep()
endif
return 1		// means msg has been processed


METHOD DelFld() CLASS DBFDialog
if ::oFields:RecNo != 0
	ARemove(::aStruct, ::oFields:RecNo)
	::oFields:Delete()
endif
return 1		// means msg has been processed


METHOD CreateDBF() CLASS DBFDialog
local	cDBF := ::oDBFName:Text

if empty(cDBF)
	::MsgBox("Please specify the DBF Name", "DBFGen")
	return 1
endif

if len(::aFlds) == 0
	::MsgBox("Please define some fields!", "DBFGen")
	return 1
endif

if at(".", cDBF) == 0
	cDBF += ".DBF"
endif

if file(cDBF) .and. ::MsgBox("Overwrite existing file?", "DBFGen", MB_YESNO) != IDYES
	return 1
endif

dbCreate(cDBF, ::aStruct)
::MsgBox(cDBF + " Created", "DBFGen")

return 1		// means msg has been processed


METHOD OpenDBF() CLASS DBFDialog
local	cDBF := ::oDBFName:Text

if empty(cDBF)								;
.and. (cDBF := GetOpenFileName(::hWnd, "*.dbf", "Select database")) == nil
	return 1
endif

::oDBFName:Text = cDBF

use (cDBF) new shared readonly

::oFields:Items := ::aFlds := {}
aeval(::aStruct := dbStruct(),						;
      {|a| ::oFields:Add(a[DBS_NAME] + TAB				;
                         + a[DBS_TYPE] + TAB				;
                         + ltrim(str(a[DBS_LEN])) + TAB			;
                         + ltrim(str(a[DBS_DEC])))})
close

return 1		// means msg has been processed


METHOD MenuSetup() CLASS MainWindow

MENU IN self
    POPUP "&File"
	MENUITEM "&New DBF..."	COMMAND DBFNew	HELP WHelp{"Specify a DBF"}
	MENUITEM SEPARATOR
	MENUITEM "E&xit"	COMMAND Exit	HELP WHelp{"Exit the application"}
    ENDPOPUP
    POPUP "&Help"
	MENUITEM "&About"					 	  ;
	         HELP WHelp{"About this application"}			  ;
	         ACTION {|| MessageBox( , "Demo written by John Skelton", ;
		                       "About", MB_ICONEXCLAMATION + MB_OK) }
    ENDPOPUP
ENDMENU

return self

