//	MDIWBrow.prg - an MDI WBrowse(s) application
//
//	Written by:	John M. Skelton,  Jun-94.
//
//	Copyright (C) 1994 Skelton Software, Kendal Cottage, Hillam, Leeds LS25 5HP, UK.
//	All Rights Reserved.
//
//	Part of Clip-4-Win.
//
//	Note: To keep this example simple, it has all the classes in this
//	      one file, and uses very simple classes.  The idea is that you
//	      can see the important things all here, but it does mean that
//	      some use of MESSAGE ... METHOD was needed.
//
//	Alternatives
//	============
//
//	You can compile this with or without MI (multiple inheritance).
//	Either:
//			clipper mdiwbrow /n /w
//	or:
//			clipper mdiwbrow /n /w /dAVOIDMI
//
//	They're not quite equivalent: the first makes the browse itself
//	the MDI child's :oClient, which means that it automatically gets
//	re-sized if the child does.  And, the re-sizing allows for any
//	toolbar / status bar (see framewin.prg).  The non-MI version just
//	uses the child itself, which is all this sample actually needs.

#include "windows.ch"
#include "topclass.ch"
#include "commands.ch"


procedure main()
local	oApp

oApp := BrowseApp{"Clip-4-Win MDI WBrowse(s) Sample"}
oApp:oWnd:MenuSetup()

oApp:Start()

oApp:Quit()

return


CLASS BrowseApp INHERIT WMDIApp
	METHOD Create(cText)	INLINE ::oWnd := MainWindow{self, cText}
ENDCLASS


CLASS MainWindow INHERIT WMDIFrame
	METHOD BrowseNew()						;
	       INLINE local	cFile,					;
	              cFile := GetOpenFileName(::hWnd, "*.dbf",		;
	                                       "Select a database"),	;
	              iif(cFile == nil, ,				;
	                  ::CreateChild(Browser{::Client, cFile})),	;
	              0

	METHOD Exit()			INLINE ::oApp:Quit()

	METHOD Init(oApp, cText)					;
	       INLINE super:Init(oApp, cText),				;
	              ::StatusBar := WStatusBar{self},			;
	              self

	METHOD MenuSetup()
ENDCLASS


CLASS Browser INHERIT WMDIChild

	PROTECT cFile	AS STRING
#ifdef	AVOIDMI
	PROTECT oBrow	AS OBJECT // AS WBrowse
#endif	// AVOIDMI
	PROTECT oTable	AS OBJECT // AS WTable

	METHOD Browse()
	METHOD Close()			INLINE SendMessage(::hWnd, WM_CLOSE)
	METHOD Create(cTitle)						;
	       INLINE super:Create(cTitle):MenuSetup():Browse()
	METHOD Init(oParent, cFile)					;
	       INLINE ::cFile := cFile,					;
	              super:Init(oParent)
	MESSAGE MenuSetup()		METHOD BrowserMenu()
	METHOD OnNCDestroy()						;
	       INLINE ::oTable:Axit(),					;
	              super:OnNCDestroy()
#ifdef	AVOIDMI
	METHOD OnSetFocus()						;
	       INLINE iif(::oBrow == nil, , SetFocus(::oBrow:hWnd)), 0
	METHOD OnSize(nType, nWidth, nHeight)				;
	       INLINE iif(::oBrow == nil, ,				;
	                  MoveWindow(::oBrow:hWnd,			;
	                             0, 0, nWidth, nHeight, .t.)),	;
	              ; // This example uses the entire child for the browse,
	              ; // so it uses MoveWindow() above, and doesn't need to
	              ; // call super, but if you had a toolbar you'd want to
	              ; // make the browse be the child's :oClient, and super
	              ; // would size everything for you.
	              super:OnSize(nType, nWidth, nHeight)
#endif	// AVOIDMI
ENDCLASS


#ifdef	AVOIDMI
#else
CLASS BrowseClient INHERIT WWindow, WBrowse
	METHOD Init(oParent, nLeft, nTop, nWidth, nHeight, cTitle, nStyle) ;
	       INLINE ; // Not wanted: (WBrowse makes its own window)
	              ; //     super( WWindow ):Init(oParent),
	              ::oParent := oParent,				;
	              super( WBrowse ):Init(oParent:hWnd,		;
	                                    nLeft, nTop, nWidth, nHeight,;
	                                    cTitle, nStyle),		;
	              self
ENDCLASS
#endif	// AVOIDMI


METHOD Browse() CLASS Browser
local	aFields			// fields/codeblocks for browse
local	oB, i
local	a := GetClientRect(::hWnd)
local	nLeft := a[1], nTop := a[2], nWidth := a[3], nHeight := a[4]

::oTable := WTable{self, ::cFile}

aFields := LdDbrowFlds()
::Text  := "Clip-4-Win Browse - " + ::oTable:Name

#ifdef	AVOIDMI
oB      := ;
::oBrow := ;
WBrowse{::hWnd, nLeft, nTop, nWidth, nHeight, "", ;
        WS_CHILD + WS_HSCROLL + WS_VSCROLL}
        // + WS_VISIBLE is assumed by WBrowse!
#else
oB      := ;
::oClient := ;
BrowseClient{self, nLeft, nTop, nWidth, nHeight, "", ;
        WS_CHILD + WS_HSCROLL + WS_VSCROLL}
        // + WS_VISIBLE is assumed by WBrowse!
#endif	// AVOIDMI

oB:Alias  := ::oTable:Alias
oB:Escape := .f.

for i = 1 to len(aFields)
	oB:AddColumn( WBColumn{aFields[i, 1], aFields[i, 2]} )
next i

SetFocus(oB:hWnd)
oB:GoTop()

return self


// LdDBrowFlds() - Returns an array for use by WBrowse{} that contains
//                 ALL fields in a database (except MEMO)

STATIC FUNCTION LdDBrowFlds()
       LOCAL nCtr
       LOCAL aFldList := {}
       LOCAL aStruct  := DBSTRUCT()

       FOR nCtr := 1 TO LEN(aStruct)
           DO CASE
              CASE aStruct[nCtr][2] == 'C'
                   AADD(aFldList, {aStruct[nCtr][1], Compile(aStruct[nCtr][1])})
              CASE aStruct[nCtr][2] == 'N'
                   AADD(aFldList, {aStruct[nCtr][1], Compile("STR("+aStruct[nCtr][1]+")")})
              CASE aStruct[nCtr][2] == 'L'
                   AADD(aFldList, {aStruct[nCtr][1], Compile("IIF("+aStruct[nCtr][1]+",'.T.','.F.')")})
              CASE aStruct[nCtr][2] == 'D'
                   AADD(aFldList, {aStruct[nCtr][1], Compile("DTOC("+aStruct[nCtr][1]+")")})
              CASE aStruct[nCtr][2] == 'M'
                   // Memo fields not supported!
           ENDCASE
         NEXT
RETURN(aFldList)

STATIC FUNCTION Compile(cStr)
RETURN(&("{||"+cStr+"}"))


METHOD MenuSetup() CLASS MainWindow

MENU IN self
    POPUP "&File"
	MENUITEM "&Browse..."	COMMAND BrowseNew  HELP WHelp{"Browse a file..."}
	MENUITEM SEPARATOR
	MENUITEM "E&xit"	COMMAND Exit	HELP WHelp{"Exit the application"}
    ENDPOPUP
    MDIWINDOWPOPUP	// provides Window Cascade, Tile, Arrange Icons, etc.
    POPUP "&Help"
	MENUITEM "&About"					 	;
	         HELP WHelp{"About this application"}			;
	         ACTION {|| MessageBox( , "Demo written by John Skelton", ;
		                       "About", MB_ICONEXCLAMATION + MB_OK) }
    ENDPOPUP
ENDMENU

return self


METHOD BrowserMenu() CLASS Browser

MENU IN self
    POPUP "&File"
	MENUITEM "&Browse..."	COMMAND BrowseNew  HELP WHelp{"Browse a file..."}
	MENUITEM SEPARATOR
	MENUITEM "&Close"	COMMAND Close	HELP WHelp{"Close browse window"}
	MENUITEM SEPARATOR
	MENUITEM "E&xit"	COMMAND Exit	HELP WHelp{"Exit the application"}
    ENDPOPUP
    POPUP   "&View"
	MENUITEM "&Your item"  ACTION {|| nil}  HELP WHelp{"Your menu item here..."}
    ENDPOPUP
    MDIWINDOWPOPUP	// provides Window Cascade, Tile, Arrange Icons, etc.
    POPUP "&Help"
	MENUITEM "&About"					 	;
	         HELP WHelp{"About this application"}			;
	         ACTION {|| MessageBox( , "Demo written by John Skelton", ;
		                       "About", MB_ICONEXCLAMATION + MB_OK) }
    ENDPOPUP
ENDMENU

return self

