/*
   PrintFls.prg
   
 - Printing any files of any size - as long as your machine has from 
   64-128k memory available.

 - Files with or without Carrage Return/Linefeeds. 
 - Any available font/color
   
 - Line width capacity of 64k, 
   
 - More speed than MemoRead()/MemoLine() routines., it takes 
   about the same time as typing the source file to the screen 
   from Dos prompt.

 - You get all with one Function call.  

   Created for use with Clip-4-Win
   Copyright (c) 1994 - 1995  Robert V. Ramsey.
   All rights reserved.
   Compile switches:  /n

   Then be sure to link PrintFls.OBJ into your Clip-4-Win application!

   All Clip4Win users are welcome to use the code at no charge as long    
   I am not held responsible for any damage whatsoever you or anyone 
   else may incur from its use.        

   If you think this program is usefull, and if you have any comments,
   you can contact me at this time:
   Phone: (206) 479-2699

*/
#Define WIN_WANT_EN                 // optional Edit NotIfication messages
#Define WIN_WANT_CLIPBOARD          // optional Clipboard constants
#Define WIN_WANT_GETDEVCAPS         // For LOGPIXELSY
#Define WIN_WANT_LBS                // to make Listbox styles accessible
#Define WIN_WANT_HELP               // For WinHelp()

//Partial listing from Windows.ch without defining WIN_WANT_ALL.
#Define WM_SETTEXT 12
#Define WM_GETTEXT 13
#Define WM_GETTEXTLENGTH 14
#Define COMMDLG_UPPERCASE

// Need to be placed after the DEFINES OtherWise the required definitions
// within the .CH files are not picked up.
#Include "Commdlg.ch"
#Include "Directry.ch"
#Include 'Fileio.ch'
#Include "TextMetr.ch"
#Include "Windows.ch"
 
//Defines for Printer Program.
#Define MAX_WIDTH 64000
#Define MAX_SUB   4095     //Probably wont happen; however, could cause
                           //a runtime crash. better to lose some print data
                           //than corrupt memory, in my opinion.
#Define _CRLF Chr(13)+Chr(10)
#Define _CR   Chr(13)
#Define _LF   Chr(10)
#Define _FF   Chr(12)

/*  
  This program is called with (hWnd) passed to it.
  
  --Some line of code..
  PrintDiskFile(hWnd)
  --Another other line of code..
*/
*---------------------------
Function PrintDiskFile(hWnd)
*---------------------------
Local aFilter := {{'Text Fls (*.TXT;*.CSV;*.DOC)','*.txt;*.csv;*.doc'},;
                  {'Source (*.C;*.COB;*.PRG)','*.c;*.cob;*.prg'},;
                  {'Progs (*.BAT;*.COM;*.EXE)','*.bat;*.com;*.bat'},;
                  {'All Fls (*.*)','*.*'}}
Local StrtLine:=Sub:=x:=y:=z:=i:=nLineWdth:=SourceLen:=nLines:=0
Local nBytesRd:=0
Local PrintLine
Local cFile
Local fHandle
Local cPrnFile
Local aPrintDlg := Array(PD_LENGTH)   // from COMMDLG.CH
Local hCursor
Local hOldcursor
Local hPrintFont
Local Oldheight
Local nPointSize:=10
Local aFont
Local hLib
Local aTm := {}
Local nChrHeight
Local nChrsPerLine, nLinesPerPage
Local hOldFont
Local PD_vars, DoPageNumb := DoCollate := .f.
Local nCurrLine, nFirstLineToPrint
Local MsgText
Local aLeftOver:={}
Local aParsed:={}
Local aSource:={}                   

cFile:=GetOpenFile(hWnd,aFilter[1,2],'Select a file for printing',;
                   aFilter,,CurDir(),'*.*')

If cFile == NIL
   // cancelled by the user
    Return
EndIf

If aFont == NIL
   //Start Font out as Fixed font so that if uses tries to print a report
   //the columns will align properly ,and not shift left; otherwise, user
   //and or programmer may never get it right.
   aFont := {0,0, 0, 0, 400, .f., .f., .f., 1, 0, 0, 0, 0, "Courier New"}
EndIf

// allows user to input FROM.. TO.. Page numbers up to MaxNm.
// If this is not initialized user will not be able to enter 
// numbers as the WINDOWS default For PD_NMAXPAGE is 0 which 
// prevents  the user from entering something bigger than 0.

aPrintDlg[PD_NMINPAGE] := 1
aPrintDlg[PD_NMAXPAGE] := 99999 

aPrintDlg[PD_NFROMPAGE] := 1
aPrintDlg[PD_NTOPAGE] := 99999 

// set up initial values For printer dialog
// see COMMDLG.CH For other flags that can be used
aPrintDlg[PD_HWNDOWNER] := SelectWindow()
aPrintDlg[PD_NFLAGS]    := PD_NOSELECTION + PD_RETURNDC 
PD_vars := aPrintDlg[PD_NFLAGS]
x := PrintDlg(aPrintDlg)
If x == NIL .or. ! x
   // cancelled by the user
   Return
EndIf
// Checks If Collate was selected
DoCollate := iif(C4W_And(aPrintDlg[PD_NFLAGS],PD_COLLATE)!=0,.t.,.f.)

// save current font height
oldheight := aFont[TM_Height]

// convert point size from x/72 units to printer pixel units
aFont[TM_Height]:= -(nPointsize * ;
                     GetDeviceCaps(aPrintDlg[PD_HDC], LOGPIXELSY) / 72)

x:=StartDoc(aPrintDlg[PD_HDC],Upper(cFile))

If x < 0
   Do Case
   Case x=-1
     MsgText:='Spooler error'
   Case x=-2
     MsgText:='Application abort'
   Case x=-4
     MsgText:='Out of disk space'
   Case x=-5
     MsgText:='Out of memory'
   EndCase
   If x<>-3
      MessageBox(0, MsgText,"Spooler message", MB_OK + MB_ICONASTERISK)
   EndIf
   Return
EndIf
Answer:=MessageBox(0, "Automatic page numbering?", ;
                 "Page Numbers", MB_YESNOCANCEL + MB_ICONQUESTION)
If Answer==IDCANCEL
   Return
EndIf
If Answer==IDYES
   DoPageNumb:=.t.
Else
   DoPageNumb:=.f.
EndIf
/*
   Select the best matching font.
   This could be selected elsewhere and passed down to this program.
*/
aFont:=ChooseFont(aFont,@nPointSize)
If aFont == NIL
    // cancelled by the user
    Return
EndIf
hPrintFont := CreateFont(aFont)
hOldFont := SelectObject(aPrintDlg[PD_HDC], hPrintFont)
GetTextMetrics(aPrintDlg[PD_HDC], @aTm)
nChrHeight    := aTm[TM_Height] + aTm[TM_ExternalLeading]
nChrsPerLine  := C4W_Int(GetDeviceCaps(aPrintDlg[PD_HDC], HORZRES) / aTm[TM_AveCharWidth])
nLinesPerPage := C4W_Int((GetDeviceCaps(aPrintDlg[PD_HDC], VERTRES) / nChrHeight))-5
nLinesPerPage +=iif(DoPageNumb,0,3)

hCursor    := LoadCursor(_GetInstance(),IDC_WAIT)
hOldCursor := SetCursor(hCursor)

nLineWdth:=iif(nChrsPerLine<=MAX_WIDTH,nChrsPerLine,MAX_WIDTH)
SourceLen:=Directory(cFile)[1][F_SIZE]
nCopies:=aPrintDlg[PD_NCOPIES]
nFromPage:=aPrintDlg[PD_NFROMPAGE]
nToPage:=aPrintDlg[PD_NTOPAGE]

/*
  The program generates as fast as typing the file at the dos prompt; 
  however, if you select a laser the actual textout() takes much longer 
  than text-printer output. If you print to a file you will see that there 
  are several lines of printer information for each actual line of print.. 
  That is why all postscript reports in any program that I know of take 
  an eternity.
*/

For xx := 1 to nCopies
   aLeftOver:={}
   aParsed:={}
   aSource:={}                   
   nBytesRd:=0
   fHandle:=fOpen(cFile,FO_READ + FO_DENYWRITE)
   Code:=fError()
   If  Code !=0 .or. fHandle=-1
       Code:=iif(Code==0,-1,Code)
       DisplayFerror(Code)
       fClose(fHandle)
       Return
   EndIf
   nCurrLine := ((nFromPage-1)*nLinesPerPage)
   If nFromPage > 1
       y:=1
       nPage:=1
       Do While .t. 
          If nPage >= nFromPage
             Exit
          EndIf
          If Len(aParsed)==0
             GetDiskPage(SourceLen,fHandle,nLineWdth,nLines,nLinesPerPage,;
                         @nBytesRd,@aParsed,@aSource)
             If Len(aParsed) < nLinesPerPage
                aParsed:={}
                Exit 
             EndIf
          EndIf
          If Left(lTrim(aParsed[1]),1)==_FF          
             aDel(aParsed,1)
             aSize(aParsed,Len(aParsed)-1)
             nPage++
             y:=1
          Else
             y++
             IF y >= nLinesPerPage
                nPage++
                y:=1
             EndIf
             aDel(aParsed,1)
             aSize(aParsed,Len(aParsed)-1)
          EndIf
       EndDo
   EndIf
   Sub:=1
   For y:= nFromPage To nToPage
       If Len(aParsed)-Sub < nLinesPerPage
          GetDiskPage(SourceLen,fHandle,nLineWdth,nLines,nLinesPerPage,;
                      @nBytesRd,@aParsed,@aSource)
          Sub:=1
       EndIf
       StartPage(aPrintDlg[PD_HDC])
       For z:= 1 to nLinesPerPage 
          If Sub > Len(aParsed)
             PrintLine:=Space(1)
          Else
             PrintLine:=aParsed[Sub]
          EndIf
          //Page break..
          If Left(lTrim(PrintLine),1)==_FF
              nn:=z
              For ii:= NN to nLinesPerPage
                 If (nCurrLine <= Len(aParsed))
                     TextOut(aPrintDlg[PD_HDC],0,(NN*nChrHeight),Space(1))
                     nCurrLine++
                 EndIf
              Next
              z:=nLinesPerPage
          Else
              TextOut(aPrintDlg[PD_HDC],0,(z*nChrHeight),PrintLine)
              nCurrLine++
          EndIf
          Sub++
       Next
       If DoPageNumb
           //Print an empty line + a line w/the Page Number
           PgStr:="Page "+AllTrim(Str(y))
           PgLFt:=Round((nChrsPerLine-Len(PgStr))/2,0)
           TextOut(aPrintDlg[PD_HDC],0,((nLinesPerpages+3)*nChrHeight),;
                   Space(PgLFt)+PgStr)
       EndIf
       EndPage(aPrintDlg[PD_HDC])
       // after an endpage you need to re-select the object in use
       SelectObject(aPrintDlg[PD_HDC], hPrintFont)
       aLeftOver:={}
       If Len(aParsed)-Sub < nLinesPerPage 
          If Sub < Len(aParsed)
             aLeftOver:={}
             For nn:= Sub To Len(aParsed)
                If Len(aLeftOver)<=MAX_SUB
                   aAdd(aLeftOver,aParsed[nn])
                EndIf
             Next
             aParsed:={}
             aParsed:=aLeftOver
             aLeftOver:={}
          Else
             aParsed:={}
          EndIf
       EndIf
       If nBytesRd >= SourceLen .and. Len(aParsed)==0
          y:=nToPage
       EndIf
   Next
   If DoCollate
      StartPage(aPrintDlg[PD_HDC])
      EndPage(aPrintDlg[PD_HDC])
      SelectObject(aPrintDlg[PD_HDC], hPrintFont)
   EndIf
   fClose(fHandle)
Next
EndDoc(aPrintDlg[PD_HDC])
SetCursor(hOldCursor)
SelectObject(aPrintDlg[PD_HDC], hOldFont)
DeleteObject(hPrintFont)
DeleteDC(aPrintDlg[PD_HDC])
Release fHandle
Release aFilter,StrtLine,PrintLine,cFile,cPrnFile,aPrintDlg,hCursor
Release hOldcursor,hPrintFont,Oldheight,nPointSize,aFont,x,y,z,i,hLib,aTm
Release nChrHeight,nChrsPerLine,nLinesPerPage,hOldFont,PD_vars 
Release DoPageNumb,DoCollate,nCurrLine,nFirstLineToPrint,MsgText
Release aParsed,SourceLen
Return
*-------------------------------------------------------------
Function ParseDiskFile(nLineWdth,nLinesPerPage,aSource,aParsed)
*-------------------------------------------------------------
Local nPos:=oPos:=0
Local ii:=1
Temp:={}
For ii:=1 To Len(aSource)
   For x:=1 To Len(aSource[ii]) Step nLineWdth
      If Len(Temp)<=MAX_SUB
         aAdd(Temp,SubStr(aSource[ii],x,nLineWdth))
      EndIf
   Next
Next
aSource:={}
For ii:=1 to Len(Temp)
   aAdd(aSource,Temp[ii])
Next
If Left(aSource[1],1)==_LF
   aSource[1]:=Right(aSource[1],Len(aSource[1])-1)
EndIf
Temp:={}
ii:=1
Do While .t.
   If Len(aParsed) > 0      
      WrapInputLine(nLineWdth,@aParsed)
   EndIf
   ParseFormFeed(@aParsed)
   WrapInputLine(nLineWdth,@aParsed)
   nPos:=At(_CRLF,aSource[ii])
   If nPos > 0
      If Left(lTrim(aSource[ii]),2)!=_CRLF .and. !Empty(aParsed)
         aParsed[Len(aParsed)]:=aParsed[Len(aParsed)]+Left(aSource[ii],nPos-1)
         WrapInputLine(nLineWdth,@aParsed)
         ParseFormFeed(@aParsed)
         If SubStr(aSource[ii],nPos,2)==_CRLF
            If Len(aParsed)<=MAX_SUB
               aAdd(aParsed,"")
            EndIf
         EndIf
      Else
         If Len(aParsed)<=MAX_SUB
            Aadd(aParsed,Left(aSource[ii],nPos-1))
         EndIf
         WrapInputLine(nLineWdth,@aParsed)
         ParseFormFeed(@aParsed)
         If SubStr(aSource[ii],nPos,2)==_CRLF
            If Len(aParsed)<=MAX_SUB
               aAdd(aParsed,"")
            EndIf
         EndIf
      EndIf
      aSource[ii]:=SubStr(aSource[ii],nPos+2,Len(aSource[ii])-nPos+2)
      Loop
   Else
      If Right(rTrim(aSource[ii]),1)==_CR 
         If (ii+1) <= Len(aSource)  
            nPos:=At(_CR,aSource[ii])
            oPos:=At(_LF,aSource[ii+1])
            //--trim() removes _LF ,but not _CR ??
            If oPos==1
               aSource[ii]:=SubStr(aSource[ii],1,nPos-1)+;
                           SubStr(aSource[ii],nPos+1,Len(aSource[ii])-1) 
               aSource[ii+1]:=_CR+aSource[ii+1]
               Loop
            EndIf
         Else
            nPos:=At(_CR,aSource[ii])
            aSource[ii]:=Left(aSource[ii],nPos-1)
         EndIf
      Else
        If Len(aParsed) == 0
           aAdd(aParsed,"")
        EndIf
      EndIf
   EndIf
   If Len(aParsed) > 0
      aParsed[Len(aParsed)]:=aParsed[Len(aParsed)]+aSource[ii]
      WrapInputLine(nLineWdth,@aParsed)
      ParseFormFeed(@aParsed)
   EndIf
   ParseFormFeed(@aParsed)
   WrapInputLine(nLineWdth,@aParsed)
   ii++
   aSource[ii-1]:=""  //Save as much Memory as you can.
   If ii > Len(aSource)
      Exit
   EndIf
EndDo
Release nPos,oPos,ii
return
*-------------------------------
Function ParseFormFeed(aParsed)
*-------------------------------
Local nPos:=0   
Local cBuff
If Len(aParsed) > 0
   nPos:=At(_FF,aParsed[Len(aParsed)])
   cBuff:=aParsed[Len(aParsed)]
   If nPos > 1
      aParsed[Len(aParsed)]:=Left(cBuff,nPos-1) 
      If Len(aParsed)<=MAX_SUB
         aAdd(aParsed,_FF)
      EndIf
      If Len(aParsed)<=MAX_SUB
         aAdd(aParsed,Right(cBuff,Len(cBuff)-nPos))
      EndIf
   EndIf
   If nPos==1
      aParsed[Len(aParsed)]:=Left(cBuff,1) 
      If Len(aParsed)<=MAX_SUB
         aAdd(aParsed,Right(cBuff,Len(cBuff)-nPos))
      EndIf
   EndIf
EndIf 
Release nPos,cBuff
Return 
*----------------------------------------
Function WrapInputLine(nLineWdth,aParsed)         
*----------------------------------------
Local cDiff:=0
If Len(aParsed) > 0
   If Len(aParsed[Len(aParsed)]) > nLineWdth
      cDiff:=Len(aParsed[Len(aParsed)])-nLineWdth
      If Len(aParsed)<=MAX_SUB
         aAdd(aParsed,"")
      EndIf
      aParsed[Len(aParsed)]:=Right(aParsed[Len(aParsed)-1],cDiff)
      aParsed[Len(aParsed)-1]:=Left(aParsed[Len(aParsed)-1],;
         Len(aParsed[Len(aParsed)-1])-cDiff)
   EndIf
EndIf
Release cDiff
Return
*-----------------------------------------------------------------------------
Function DiskReadFile(SourceLen,fHandle,nLineWdth,nLines,;
         nLinesPerPage,nBytesRd,aParsed,aSource)
*-----------------------------------------------------------------------------
/*   
   FreadStr() will not return Chr(0), Some files have many machine
   bytes, Chr(0) will cause cBuff to lose some characters because of 
   dos pointer etc..., it really shouldn't cause much of a problem.  
   There is nothing that I know of that can be done.  You would get 
   garbage from other generic print utilities as well, if you
   try to print software specific files.
*/
Local ii:=1
Local nPos:=0
aSource:={}
ii:=1
If nBytesRd < SourceLen
   Do While .t.
      cBuff:=fReadStr(fHandle,1360)
      nBytesRd+=1360
      If cBuff=="" .and. nBytesRd >= SourceLen
          Exit
      Else
         aAdd(aSource,cBuff)
         If ii >= nLinesPerPage
            Exit
         EndIf
      EndIf
      ii++
   EndDo
EndIf
Release nPos,ii
return
*----------------------------------------------------------------------
Function GetDiskPage(SourceLen,fHandle,nLineWdth,nLines,;
             nLinesPerPage,nBytesRd,aParsed,aSource)
*----------------------------------------------------------------------
/*   
   Soft wrapping in source file is the fly in the soup.  Appending from
   a file without hard returns isn't handled by 'Append SDF' very well.
   One could use a database rather than arrays, but database fields are
   are fixed-length and don't allow the real world baloon like flexability
   of unknown strings without causing several kinds of problems.

   The parsing engine of this program is much faster than using 
   Memoread()/MemoLine().  I have tested it and found that I cant 
   make it print any faster.  It takes approximately as long to type
   a file to the screen at Dos, as it takes for this program 
   to print it, to the device handle.

   In theory you should be able to print a file larger than your
   available memory, when using this parsing engine.
*/
aSource:={}
DiskReadFile(SourceLen,fHandle,nLineWdth,nLines,nLinesPerPage,;
             @nBytesRd,@aParsed,@aSource)
If Len(aSource) > 0
   ParseDiskFile(nLineWdth,nLinesPerPage,@aSource,@aParsed)
EndIf
Return
*---------------------------
Function DisplayFerror(Code)
*---------------------------
Local cString
   Do Case     
   Case Code==2
     cString:='File not found.'
   Case Code==3
     cString:='Path not found.'
   Case Code==4
     cString:='Too many files open.'
   Case Code==5
     cString:='Access denied.' 
   Case Code==6
     cString:='Invalid handle.' 
   Case Code==8
     cString:='Insufficient memory.' 
   Case Code==15
     cString:='Invalid drive specified.' 
   Case Code==19
     cString:='Attempted to write to a write-protected disk.'
   Case Code==21
     cString:='Drive not ready.' 
   Case Code==23
     cString:='Data CRC error.' 
   Case Code==29
     cString:='Write fault.' 
   Case Code==30
     cString:='Read fault.' 
   Case Code==32
     cString:='Sharing violation.' 
   Case Code==33
     cString:='Lock Violation.'
   OtherWise
     cString:='Unknown problem with file.'
   EndCase
   cString:='Cannot print file, because: '+cString
   MessageBox(0, cString,"Error with file", MB_OK + MB_ICONASTERISK)
Release cString
Return
