/*
 *	offset.c - support routine for Clip-4-Win
 *
 *	Written by:	John M. Skelton,  Mar-93.
 *	Modified by:	John M. Skelton,  Feb-94.  Added lengthy comments.
 *
 *	Copyright (C) 1993,1994 Skelton Software, Kendal Cottage, Hillam, Leeds LS25 5HP.
 *	All Rights Reserved.
 */


// This is all that's actually required for this tiny sample:
//
//	#define OffsetRect _OffsetRect
//
// but the generic way to rename functions is to #include "unwindow.h",
// which renames all the names with 10 or fewer chars when the
//	#include "windows.h"
// is done.  See readme.txt.


// Clipper's header (#include) files conflict with windows.h (note that
// 5.2 conflicts more than 5.01), so there's a generic header that has:
//
//	1. #include "unwindow.h", to rename things
//	2. #define STRICT, to encourage careful coding
//	3. #include <windows.h>
//	4. #include "extend.h", with much mess to overcome the
//	   the conflicts between 5.2 and Windows.

#include "c4w.h"


static const char	*copyright[] =
{
	"Copyright (C) 1993 Skelton Software, Kendal Cottage, Hillam, Leeds, UK.",
	"All Rights Reserved."
};


// Because C names are case-sensitive, we can just spell the function
// name with a different mix of case.  So, OFFSETRECT or OFFsetRect
// would do.
//
// The "CLIPPER" type makes the function name all upper-case, but I
// find mixed-case a little more readable.

CLIPPER OFfsetRect(void)	// (aRect, nX, nY) --> nil
{
	RECT	r;

	r.left   = _parni(1, 1);
	r.top    = _parni(1, 2);
	r.right  = _parni(1, 3);
	r.bottom = _parni(1, 4);

	// You can specify the FAR if you want, but you should
	// be compiling large model anyway.  You can use huge
	// model, but it's rarely necessary and tends to just
	// create extra overhead.
	//	OffsetRect((RECT FAR *) &r, _parni(2), _parni(3));
	OffsetRect(&r, _parni(2), _parni(3));
}
