/*  MSGBAR

Program displays microhelp (message bar) across the base of a
window and displays the x,y position of the mouse cursor as the user
moves the mouse. Demonstrates an application of the subclassing
technique. Elements derived from msgbar.prg, also in this section;
improves on it by using child window controls instead of drawn objects,
eliminating redraw concerns.

Comments welcomed by:

Basil McDonnell 74007,2034

*/

#define WIN_WANT_ALL
#define WIN_WANT_DRAWTEXT

#include "windows.ch"

static  cAppName := "msgbar"
static  debug_sub,parenth

function main()
local   childhwnd, hwnd, Newhwnd , hinst, nEvent, hPal, hMSGlib, cDisable, nProc

hWnd = WinSetup(cAppName, "MsgBar")
hInst = _GetInstance()

newhWnd := CreateWindow(cAppname,           ;       // window class
                    "", ;
                    WS_VISIBLE + WS_OVERLAPPEDWINDOW, ;
                    10,                  ;       // x co-ordinate
                    100,                  ;       // y co-ordinate
                    600,              ;       // width
                    200,             ;       // height
                    hWnd,                ;       // hWnd of parent
                    0,                   ;       // hMenu of menu (none yet)
                    hInst)                       // our own app instance

// Install message bar frame in the window
MsgBar(newhWnd)

// Subclass the window.
// Note the last parameter is an array that indicates the messages
// that Windows will pass through to the subclassed window.  In this
// example, only mousemove is being allowed through.

nProc = SubClassWindow(newhWnd,                                            ;
                     {|hWnd, nMsg, nwParam, nlParam|                  ;
                      ShowParms(nProc, hWnd, nMsg, nwParam, nlParam)},  ;
                      {WM_MOUSEMOVE})

do while .t.
   DO WHILE (nEvent := ChkEvent()) == EVENT_NONE
   ENDDO
   do case
   case nEvent == EVENT_QUIT
        quit
   endcase
enddo
return nil

/*-------------------------------------------------------------------------
Subclass the window to this procedure;
fills the message bar with the current mouse position
--------------------------------------------------------------------------*/
function ShowParms(nProc, hWnd, nMsg, nwParam, nlParam)
static	hBrush
local	nRet

message( hwnd, ltrim(str(mousex()))+" "+ ltrim(str(mousey())) )

return CallWindowProc(nProc, hWnd, nMsg, nwParam, nlParam)

/*-------------------------------------------------------------------------
// FUNCTION Msgbar
// modified to use controls
--------------------------------------------------------------------------*/

FUNCTION MsgBar(hWnd)
         LOCAL hInst := _GetInstance(),childhwnd
         LOCAL nLeft, nTop, nRight, nBottom, aCRect, hOldPen
         LOCAL hColor, hBlackPen, hGreyPen, hWhitePen

         registerclass( CS_SAVEBITS, hinst,0,0,COLOR_BTNSHADOW,"msgbackgnd")
         registerclass( CS_SAVEBITS, hinst,0,0,COLOR_GRAYTEXT,"msggrey")

         aCRect  := GetClientRect(hWnd)
         nLeft   := 0
         nTop    := aCRect[4]-26
         nRight  := aCRect[3]
         nBottom := nTop+26

         childhwnd = CreateWindow("msggrey",         ;       // button class name
                              "",  ;       // title text
                               WS_CHILD+WS_VISIBLE, ;
                              nLeft, nTop,         ;       // x,y position
                              nRight, nBottom,          ;       // width, height
                              hwnd,             ;       // parent window
                              0, ;
			      hInst)			// parent's instance

         childhwnd = CreateWindow("msgbackgnd",         ;       // button class name
                              "",  ;       // title text
                               WS_CHILD+WS_VISIBLE, ;
                              nLeft, nTop+1,         ;       // x,y position
                              nRight, 26,          ;       // width, height
                              hwnd,             ;       // parent window
                              0, ;
			      hInst)			// parent's instance

         childhwnd = CreateWindow("static",         ;       // button class name
                              "",  ;       // title text
                               WS_CHILD+WS_VISIBLE+SS_BLACKFRAME, ;
                              nLeft+10, nTop+4,         ;       // x,y position
                              nRight-20, 18,          ;       // width, height
                              hwnd,             ;       // parent window
                              0, ;
			      hInst)			// parent's instance

         childhwnd = CreateWindow("static",         ;       // button class name
                              "",  ;       // title text
                               WS_CHILD+WS_VISIBLE+SS_WHITEFRAME, ;
                              nLeft+10 , nTop+4+17,         ;       // x,y position
                              nRight-20, 2,          ;       // width, height
                              hwnd,             ;       // parent window
                              0, ;
			      hInst)			// parent's instance

         childhwnd = CreateWindow("static",         ;       // button class name
                              "",  ;       // title text
                               WS_CHILD+WS_VISIBLE+SS_WHITEFRAME, ;
                              nLeft+ 9 + (nRight-20), nTop+4,         ;       // x,y position
                              2, 18,          ;       // width, height
                              hwnd,             ;       // parent window
                              0, ;
			      hInst)			// parent's instance


RETURN(NIL)

/*-------------------------------------------------------------------------
// Message: no redraw required, background is automatic
// ... and I like my messages on the left...
--------------------------------------------------------------------------*/
FUNCTION Message(hWnd,cText)
         LOCAL nLeft, nTop, nRight, nBottom, aCRect, hDC
         LOCAL nOldBkClor

         aCRect  := GetClientRect(hWnd)
         nLeft   := 0
         nTop    := aCRect[4]-26
         nRight  := aCRect[3]  // New for this version
         nBottom := nTop+28

         hDC        := GetDC(hWnd)
         nOldBkClor := SetBkColor(hDC,RGB(192,192,192))

         DrawText(hDC,space(30),;
                     {nLeft+20,nTop+5,nRight-11,nBottom-5},DT_LEFT)
         DrawText(hDC,cText,;
                     {nLeft+20,nTop+5,nRight-11,nBottom-5},DT_LEFT)
         SetBkColor(hDC,nOldBkClor)
         ReleaseDC(hWnd,hDC)
RETURN(NIL)

/*-------------------------------------------------------------------------
// Standard Winsetup (Unmodified from demo source)
--------------------------------------------------------------------------*/
static Function WinSetup(cAppName,cTitle)
local   hWnd,hInst,hPrevinst,nCmdShow

/////////////////////////
//
//	Typical windows-style setup code...
//
/////////////////////////

hInst = _GetInstance()
hPrevInst = _GetPrevInstance()
nCmdShow = _GetnCmdShow()

if hPrevInst == 0
	// need to register the window class
	if !RegisterClass(CS_HREDRAW + CS_VREDRAW,	;
			  hInst,	;
			  0,		;	// default icon
			  0,		;	// default cursor
			  0,		;	// default background brush
			  cAppName)		// app name

		// probably out of resources
		quit
	endif
endif

// create our main window
hWnd = CreateWindow(cAppName,		;	// window class
                    cTitle, ;                   // caption for title bar
                    WS_OVERLAPPEDWINDOW + WS_HSCROLL,;       // window style
                    0,      ;       // x co-ordinate
                    0,                  ;       // y co-ordinate
                    700,                ;       // width
                    500,                ;       // height
		    0,			;	// hWnd of parent (none)
		    0,			;	// hMenu of menu (none yet)
		    hInst)			// our own app instance

if hWnd == 0
	// probably out of resources
	quit
endif

// make sure it's displayed ...
ShowWindow(hWnd, nCmdShow)

// ... and up to date
UpdateWindow(hWnd)

return hWnd

